﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SQLAzureMWUtils
{
    public class Collation
    {
        public string Name { get; set; }
        public string Description { get; set; }

        public Collation(string name, string description)
        {
            Name = name;
            Description = description;
        }

        public override string ToString()
        {
            return Description;
        }
    }

    public static class CollationHelper
    {
        private static List<Collation> _collationList;

        public static List<Collation> GetCollationList()
        {
            if (_collationList == null)
            {
                _collationList = new List<Collation>(40);
                _collationList.Add(new Collation("", "<server default>"));
                _collationList.Add(new Collation("Albanian_BIN2", "Albanian, binary code point comparison sort"));
                _collationList.Add(new Collation("Albanian_BIN", "Albanian, binary sort"));
                _collationList.Add(new Collation("Albanian_CI_AI", "Albanian, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Albanian_CI_AI_WS", "Albanian, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Albanian_CI_AI_KS", "Albanian, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Albanian_CI_AI_KS_WS", "Albanian, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Albanian_CI_AS", "Albanian, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Albanian_CI_AS_WS", "Albanian, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Albanian_CI_AS_KS", "Albanian, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Albanian_CI_AS_KS_WS", "Albanian, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Albanian_CS_AI", "Albanian, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Albanian_CS_AI_WS", "Albanian, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Albanian_CS_AI_KS", "Albanian, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Albanian_CS_AI_KS_WS", "Albanian, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Albanian_CS_AS", "Albanian, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Albanian_CS_AS_WS", "Albanian, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Albanian_CS_AS_KS", "Albanian, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Albanian_CS_AS_KS_WS", "Albanian, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Albanian_100_BIN2", "Albanian-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Albanian_100_BIN", "Albanian-100, binary sort"));
                _collationList.Add(new Collation("Albanian_100_CI_AI", "Albanian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Albanian_100_CI_AI_WS", "Albanian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Albanian_100_CI_AI_KS", "Albanian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Albanian_100_CI_AI_KS_WS", "Albanian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Albanian_100_CI_AS", "Albanian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Albanian_100_CI_AS_WS", "Albanian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Albanian_100_CI_AS_KS", "Albanian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Albanian_100_CI_AS_KS_WS", "Albanian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Albanian_100_CS_AI", "Albanian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Albanian_100_CS_AI_WS", "Albanian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Albanian_100_CS_AI_KS", "Albanian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Albanian_100_CS_AI_KS_WS", "Albanian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Albanian_100_CS_AS", "Albanian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Albanian_100_CS_AS_WS", "Albanian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Albanian_100_CS_AS_KS", "Albanian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Albanian_100_CS_AS_KS_WS", "Albanian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Arabic_BIN2", "Arabic, binary code point comparison sort"));
                _collationList.Add(new Collation("Arabic_BIN", "Arabic, binary sort"));
                _collationList.Add(new Collation("Arabic_CI_AI", "Arabic, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Arabic_CI_AI_WS", "Arabic, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Arabic_CI_AI_KS", "Arabic, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Arabic_CI_AI_KS_WS", "Arabic, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Arabic_CI_AS", "Arabic, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Arabic_CI_AS_WS", "Arabic, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Arabic_CI_AS_KS", "Arabic, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Arabic_CI_AS_KS_WS", "Arabic, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Arabic_CS_AI", "Arabic, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Arabic_CS_AI_WS", "Arabic, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Arabic_CS_AI_KS", "Arabic, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Arabic_CS_AI_KS_WS", "Arabic, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Arabic_CS_AS", "Arabic, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Arabic_CS_AS_WS", "Arabic, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Arabic_CS_AS_KS", "Arabic, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Arabic_CS_AS_KS_WS", "Arabic, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Arabic_100_BIN2", "Arabic-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Arabic_100_BIN", "Arabic-100, binary sort"));
                _collationList.Add(new Collation("Arabic_100_CI_AI", "Arabic-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Arabic_100_CI_AI_WS", "Arabic-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Arabic_100_CI_AI_KS", "Arabic-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Arabic_100_CI_AI_KS_WS", "Arabic-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Arabic_100_CI_AS", "Arabic-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Arabic_100_CI_AS_WS", "Arabic-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Arabic_100_CI_AS_KS", "Arabic-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Arabic_100_CI_AS_KS_WS", "Arabic-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Arabic_100_CS_AI", "Arabic-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Arabic_100_CS_AI_WS", "Arabic-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Arabic_100_CS_AI_KS", "Arabic-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Arabic_100_CS_AI_KS_WS", "Arabic-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Arabic_100_CS_AS", "Arabic-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Arabic_100_CS_AS_WS", "Arabic-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Arabic_100_CS_AS_KS", "Arabic-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Arabic_100_CS_AS_KS_WS", "Arabic-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Assamese_100_BIN2", "Assamese-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Assamese_100_BIN", "Assamese-100, binary sort"));
                _collationList.Add(new Collation("Assamese_100_CI_AI", "Assamese-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Assamese_100_CI_AI_WS", "Assamese-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Assamese_100_CI_AI_KS", "Assamese-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Assamese_100_CI_AI_KS_WS", "Assamese-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Assamese_100_CI_AS", "Assamese-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Assamese_100_CI_AS_WS", "Assamese-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Assamese_100_CI_AS_KS", "Assamese-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Assamese_100_CI_AS_KS_WS", "Assamese-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Assamese_100_CS_AI", "Assamese-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Assamese_100_CS_AI_WS", "Assamese-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Assamese_100_CS_AI_KS", "Assamese-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Assamese_100_CS_AI_KS_WS", "Assamese-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Assamese_100_CS_AS", "Assamese-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Assamese_100_CS_AS_WS", "Assamese-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Assamese_100_CS_AS_KS", "Assamese-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Assamese_100_CS_AS_KS_WS", "Assamese-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Azeri_Cyrillic_100_BIN2", "Azeri-Cyrillic-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Azeri_Cyrillic_100_BIN", "Azeri-Cyrillic-100, binary sort"));
                _collationList.Add(new Collation("Azeri_Cyrillic_100_CI_AI", "Azeri-Cyrillic-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Azeri_Cyrillic_100_CI_AI_WS", "Azeri-Cyrillic-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Azeri_Cyrillic_100_CI_AI_KS", "Azeri-Cyrillic-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Azeri_Cyrillic_100_CI_AI_KS_WS", "Azeri-Cyrillic-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Azeri_Cyrillic_100_CI_AS", "Azeri-Cyrillic-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Azeri_Cyrillic_100_CI_AS_WS", "Azeri-Cyrillic-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Azeri_Cyrillic_100_CI_AS_KS", "Azeri-Cyrillic-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Azeri_Cyrillic_100_CI_AS_KS_WS", "Azeri-Cyrillic-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Azeri_Cyrillic_100_CS_AI", "Azeri-Cyrillic-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Azeri_Cyrillic_100_CS_AI_WS", "Azeri-Cyrillic-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Azeri_Cyrillic_100_CS_AI_KS", "Azeri-Cyrillic-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Azeri_Cyrillic_100_CS_AI_KS_WS", "Azeri-Cyrillic-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Azeri_Cyrillic_100_CS_AS", "Azeri-Cyrillic-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Azeri_Cyrillic_100_CS_AS_WS", "Azeri-Cyrillic-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Azeri_Cyrillic_100_CS_AS_KS", "Azeri-Cyrillic-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Azeri_Cyrillic_100_CS_AS_KS_WS", "Azeri-Cyrillic-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Azeri_Latin_100_BIN2", "Azeri-Latin-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Azeri_Latin_100_BIN", "Azeri-Latin-100, binary sort"));
                _collationList.Add(new Collation("Azeri_Latin_100_CI_AI", "Azeri-Latin-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Azeri_Latin_100_CI_AI_WS", "Azeri-Latin-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Azeri_Latin_100_CI_AI_KS", "Azeri-Latin-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Azeri_Latin_100_CI_AI_KS_WS", "Azeri-Latin-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Azeri_Latin_100_CI_AS", "Azeri-Latin-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Azeri_Latin_100_CI_AS_WS", "Azeri-Latin-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Azeri_Latin_100_CI_AS_KS", "Azeri-Latin-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Azeri_Latin_100_CI_AS_KS_WS", "Azeri-Latin-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Azeri_Latin_100_CS_AI", "Azeri-Latin-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Azeri_Latin_100_CS_AI_WS", "Azeri-Latin-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Azeri_Latin_100_CS_AI_KS", "Azeri-Latin-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Azeri_Latin_100_CS_AI_KS_WS", "Azeri-Latin-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Azeri_Latin_100_CS_AS", "Azeri-Latin-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Azeri_Latin_100_CS_AS_WS", "Azeri-Latin-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Azeri_Latin_100_CS_AS_KS", "Azeri-Latin-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Azeri_Latin_100_CS_AS_KS_WS", "Azeri-Latin-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Bashkir_100_BIN2", "Bashkir-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Bashkir_100_BIN", "Bashkir-100, binary sort"));
                _collationList.Add(new Collation("Bashkir_100_CI_AI", "Bashkir-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Bashkir_100_CI_AI_WS", "Bashkir-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Bashkir_100_CI_AI_KS", "Bashkir-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Bashkir_100_CI_AI_KS_WS", "Bashkir-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Bashkir_100_CI_AS", "Bashkir-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Bashkir_100_CI_AS_WS", "Bashkir-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Bashkir_100_CI_AS_KS", "Bashkir-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Bashkir_100_CI_AS_KS_WS", "Bashkir-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Bashkir_100_CS_AI", "Bashkir-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Bashkir_100_CS_AI_WS", "Bashkir-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Bashkir_100_CS_AI_KS", "Bashkir-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Bashkir_100_CS_AI_KS_WS", "Bashkir-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Bashkir_100_CS_AS", "Bashkir-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Bashkir_100_CS_AS_WS", "Bashkir-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Bashkir_100_CS_AS_KS", "Bashkir-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Bashkir_100_CS_AS_KS_WS", "Bashkir-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Bengali_100_BIN2", "Bengali-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Bengali_100_BIN", "Bengali-100, binary sort"));
                _collationList.Add(new Collation("Bengali_100_CI_AI", "Bengali-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Bengali_100_CI_AI_WS", "Bengali-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Bengali_100_CI_AI_KS", "Bengali-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Bengali_100_CI_AI_KS_WS", "Bengali-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Bengali_100_CI_AS", "Bengali-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Bengali_100_CI_AS_WS", "Bengali-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Bengali_100_CI_AS_KS", "Bengali-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Bengali_100_CI_AS_KS_WS", "Bengali-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Bengali_100_CS_AI", "Bengali-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Bengali_100_CS_AI_WS", "Bengali-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Bengali_100_CS_AI_KS", "Bengali-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Bengali_100_CS_AI_KS_WS", "Bengali-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Bengali_100_CS_AS", "Bengali-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Bengali_100_CS_AS_WS", "Bengali-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Bengali_100_CS_AS_KS", "Bengali-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Bengali_100_CS_AS_KS_WS", "Bengali-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Bosnian_Cyrillic_100_BIN2", "Bosnian-Cyrillic-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Bosnian_Cyrillic_100_BIN", "Bosnian-Cyrillic-100, binary sort"));
                _collationList.Add(new Collation("Bosnian_Cyrillic_100_CI_AI", "Bosnian-Cyrillic-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Bosnian_Cyrillic_100_CI_AI_WS", "Bosnian-Cyrillic-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Bosnian_Cyrillic_100_CI_AI_KS", "Bosnian-Cyrillic-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Bosnian_Cyrillic_100_CI_AI_KS_WS", "Bosnian-Cyrillic-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Bosnian_Cyrillic_100_CI_AS", "Bosnian-Cyrillic-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Bosnian_Cyrillic_100_CI_AS_WS", "Bosnian-Cyrillic-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Bosnian_Cyrillic_100_CI_AS_KS", "Bosnian-Cyrillic-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Bosnian_Cyrillic_100_CI_AS_KS_WS", "Bosnian-Cyrillic-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Bosnian_Cyrillic_100_CS_AI", "Bosnian-Cyrillic-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Bosnian_Cyrillic_100_CS_AI_WS", "Bosnian-Cyrillic-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Bosnian_Cyrillic_100_CS_AI_KS", "Bosnian-Cyrillic-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Bosnian_Cyrillic_100_CS_AI_KS_WS", "Bosnian-Cyrillic-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Bosnian_Cyrillic_100_CS_AS", "Bosnian-Cyrillic-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Bosnian_Cyrillic_100_CS_AS_WS", "Bosnian-Cyrillic-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Bosnian_Cyrillic_100_CS_AS_KS", "Bosnian-Cyrillic-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Bosnian_Cyrillic_100_CS_AS_KS_WS", "Bosnian-Cyrillic-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Bosnian_Latin_100_BIN2", "Bosnian-Latin-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Bosnian_Latin_100_BIN", "Bosnian-Latin-100, binary sort"));
                _collationList.Add(new Collation("Bosnian_Latin_100_CI_AI", "Bosnian-Latin-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Bosnian_Latin_100_CI_AI_WS", "Bosnian-Latin-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Bosnian_Latin_100_CI_AI_KS", "Bosnian-Latin-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Bosnian_Latin_100_CI_AI_KS_WS", "Bosnian-Latin-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Bosnian_Latin_100_CI_AS", "Bosnian-Latin-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Bosnian_Latin_100_CI_AS_WS", "Bosnian-Latin-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Bosnian_Latin_100_CI_AS_KS", "Bosnian-Latin-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Bosnian_Latin_100_CI_AS_KS_WS", "Bosnian-Latin-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Bosnian_Latin_100_CS_AI", "Bosnian-Latin-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Bosnian_Latin_100_CS_AI_WS", "Bosnian-Latin-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Bosnian_Latin_100_CS_AI_KS", "Bosnian-Latin-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Bosnian_Latin_100_CS_AI_KS_WS", "Bosnian-Latin-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Bosnian_Latin_100_CS_AS", "Bosnian-Latin-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Bosnian_Latin_100_CS_AS_WS", "Bosnian-Latin-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Bosnian_Latin_100_CS_AS_KS", "Bosnian-Latin-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Bosnian_Latin_100_CS_AS_KS_WS", "Bosnian-Latin-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Breton_100_BIN2", "Breton-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Breton_100_BIN", "Breton-100, binary sort"));
                _collationList.Add(new Collation("Breton_100_CI_AI", "Breton-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Breton_100_CI_AI_WS", "Breton-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Breton_100_CI_AI_KS", "Breton-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Breton_100_CI_AI_KS_WS", "Breton-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Breton_100_CI_AS", "Breton-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Breton_100_CI_AS_WS", "Breton-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Breton_100_CI_AS_KS", "Breton-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Breton_100_CI_AS_KS_WS", "Breton-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Breton_100_CS_AI", "Breton-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Breton_100_CS_AI_WS", "Breton-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Breton_100_CS_AI_KS", "Breton-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Breton_100_CS_AI_KS_WS", "Breton-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Breton_100_CS_AS", "Breton-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Breton_100_CS_AS_WS", "Breton-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Breton_100_CS_AS_KS", "Breton-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Breton_100_CS_AS_KS_WS", "Breton-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Hong_Kong_Stroke_90_BIN2", "Chinese-Hong-Kong-Stroke-90, binary code point comparison sort"));
                _collationList.Add(new Collation("Chinese_Hong_Kong_Stroke_90_BIN", "Chinese-Hong-Kong-Stroke-90, binary sort"));
                _collationList.Add(new Collation("Chinese_Hong_Kong_Stroke_90_CI_AI", "Chinese-Hong-Kong-Stroke-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Hong_Kong_Stroke_90_CI_AI_WS", "Chinese-Hong-Kong-Stroke-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Hong_Kong_Stroke_90_CI_AI_KS", "Chinese-Hong-Kong-Stroke-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Hong_Kong_Stroke_90_CI_AI_KS_WS", "Chinese-Hong-Kong-Stroke-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Hong_Kong_Stroke_90_CI_AS", "Chinese-Hong-Kong-Stroke-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Hong_Kong_Stroke_90_CI_AS_WS", "Chinese-Hong-Kong-Stroke-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Hong_Kong_Stroke_90_CI_AS_KS", "Chinese-Hong-Kong-Stroke-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Hong_Kong_Stroke_90_CI_AS_KS_WS", "Chinese-Hong-Kong-Stroke-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Hong_Kong_Stroke_90_CS_AI", "Chinese-Hong-Kong-Stroke-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Hong_Kong_Stroke_90_CS_AI_WS", "Chinese-Hong-Kong-Stroke-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Hong_Kong_Stroke_90_CS_AI_KS", "Chinese-Hong-Kong-Stroke-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Hong_Kong_Stroke_90_CS_AI_KS_WS", "Chinese-Hong-Kong-Stroke-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Hong_Kong_Stroke_90_CS_AS", "Chinese-Hong-Kong-Stroke-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Hong_Kong_Stroke_90_CS_AS_WS", "Chinese-Hong-Kong-Stroke-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Hong_Kong_Stroke_90_CS_AS_KS", "Chinese-Hong-Kong-Stroke-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Hong_Kong_Stroke_90_CS_AS_KS_WS", "Chinese-Hong-Kong-Stroke-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_BIN2", "Chinese-PRC, binary code point comparison sort"));
                _collationList.Add(new Collation("Chinese_PRC_BIN", "Chinese-PRC, binary sort"));
                _collationList.Add(new Collation("Chinese_PRC_CI_AI", "Chinese-PRC, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_CI_AI_WS", "Chinese-PRC, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_CI_AI_KS", "Chinese-PRC, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_CI_AI_KS_WS", "Chinese-PRC, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_CI_AS", "Chinese-PRC, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_CI_AS_WS", "Chinese-PRC, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_CI_AS_KS", "Chinese-PRC, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_CI_AS_KS_WS", "Chinese-PRC, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_CS_AI", "Chinese-PRC, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_CS_AI_WS", "Chinese-PRC, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_CS_AI_KS", "Chinese-PRC, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_CS_AI_KS_WS", "Chinese-PRC, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_CS_AS", "Chinese-PRC, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_CS_AS_WS", "Chinese-PRC, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_CS_AS_KS", "Chinese-PRC, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_CS_AS_KS_WS", "Chinese-PRC, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_90_BIN2", "Chinese-PRC-90, binary code point comparison sort"));
                _collationList.Add(new Collation("Chinese_PRC_90_BIN", "Chinese-PRC-90, binary sort"));
                _collationList.Add(new Collation("Chinese_PRC_90_CI_AI", "Chinese-PRC-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_90_CI_AI_WS", "Chinese-PRC-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_90_CI_AI_KS", "Chinese-PRC-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_90_CI_AI_KS_WS", "Chinese-PRC-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_90_CI_AS", "Chinese-PRC-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_90_CI_AS_WS", "Chinese-PRC-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_90_CI_AS_KS", "Chinese-PRC-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_90_CI_AS_KS_WS", "Chinese-PRC-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_90_CS_AI", "Chinese-PRC-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_90_CS_AI_WS", "Chinese-PRC-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_90_CS_AI_KS", "Chinese-PRC-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_90_CS_AI_KS_WS", "Chinese-PRC-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_90_CS_AS", "Chinese-PRC-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_90_CS_AS_WS", "Chinese-PRC-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_90_CS_AS_KS", "Chinese-PRC-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_90_CS_AS_KS_WS", "Chinese-PRC-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_BIN2", "Chinese-PRC-Stroke, binary code point comparison sort"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_BIN", "Chinese-PRC-Stroke, binary sort"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_CI_AI", "Chinese-PRC-Stroke, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_CI_AI_WS", "Chinese-PRC-Stroke, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_CI_AI_KS", "Chinese-PRC-Stroke, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_CI_AI_KS_WS", "Chinese-PRC-Stroke, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_CI_AS", "Chinese-PRC-Stroke, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_CI_AS_WS", "Chinese-PRC-Stroke, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_CI_AS_KS", "Chinese-PRC-Stroke, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_CI_AS_KS_WS", "Chinese-PRC-Stroke, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_CS_AI", "Chinese-PRC-Stroke, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_CS_AI_WS", "Chinese-PRC-Stroke, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_CS_AI_KS", "Chinese-PRC-Stroke, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_CS_AI_KS_WS", "Chinese-PRC-Stroke, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_CS_AS", "Chinese-PRC-Stroke, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_CS_AS_WS", "Chinese-PRC-Stroke, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_CS_AS_KS", "Chinese-PRC-Stroke, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_CS_AS_KS_WS", "Chinese-PRC-Stroke, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_90_BIN2", "Chinese-PRC-Stroke-90, binary code point comparison sort"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_90_BIN", "Chinese-PRC-Stroke-90, binary sort"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_90_CI_AI", "Chinese-PRC-Stroke-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_90_CI_AI_WS", "Chinese-PRC-Stroke-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_90_CI_AI_KS", "Chinese-PRC-Stroke-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_90_CI_AI_KS_WS", "Chinese-PRC-Stroke-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_90_CI_AS", "Chinese-PRC-Stroke-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_90_CI_AS_WS", "Chinese-PRC-Stroke-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_90_CI_AS_KS", "Chinese-PRC-Stroke-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_90_CI_AS_KS_WS", "Chinese-PRC-Stroke-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_90_CS_AI", "Chinese-PRC-Stroke-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_90_CS_AI_WS", "Chinese-PRC-Stroke-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_90_CS_AI_KS", "Chinese-PRC-Stroke-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_90_CS_AI_KS_WS", "Chinese-PRC-Stroke-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_90_CS_AS", "Chinese-PRC-Stroke-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_90_CS_AS_WS", "Chinese-PRC-Stroke-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_90_CS_AS_KS", "Chinese-PRC-Stroke-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_PRC_Stroke_90_CS_AS_KS_WS", "Chinese-PRC-Stroke-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Pinyin_100_BIN2", "Chinese-Simplified-Pinyin-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Chinese_Simplified_Pinyin_100_BIN", "Chinese-Simplified-Pinyin-100, binary sort"));
                _collationList.Add(new Collation("Chinese_Simplified_Pinyin_100_CI_AI", "Chinese-Simplified-Pinyin-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Pinyin_100_CI_AI_WS", "Chinese-Simplified-Pinyin-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Pinyin_100_CI_AI_KS", "Chinese-Simplified-Pinyin-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Pinyin_100_CI_AI_KS_WS", "Chinese-Simplified-Pinyin-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Pinyin_100_CI_AS", "Chinese-Simplified-Pinyin-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Pinyin_100_CI_AS_WS", "Chinese-Simplified-Pinyin-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Pinyin_100_CI_AS_KS", "Chinese-Simplified-Pinyin-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Pinyin_100_CI_AS_KS_WS", "Chinese-Simplified-Pinyin-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Pinyin_100_CS_AI", "Chinese-Simplified-Pinyin-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Pinyin_100_CS_AI_WS", "Chinese-Simplified-Pinyin-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Pinyin_100_CS_AI_KS", "Chinese-Simplified-Pinyin-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Pinyin_100_CS_AI_KS_WS", "Chinese-Simplified-Pinyin-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Pinyin_100_CS_AS", "Chinese-Simplified-Pinyin-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Pinyin_100_CS_AS_WS", "Chinese-Simplified-Pinyin-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Pinyin_100_CS_AS_KS", "Chinese-Simplified-Pinyin-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Pinyin_100_CS_AS_KS_WS", "Chinese-Simplified-Pinyin-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Stroke_Order_100_BIN2", "Chinese-Simplified-Stroke-Order-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Chinese_Simplified_Stroke_Order_100_BIN", "Chinese-Simplified-Stroke-Order-100, binary sort"));
                _collationList.Add(new Collation("Chinese_Simplified_Stroke_Order_100_CI_AI", "Chinese-Simplified-Stroke-Order-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Stroke_Order_100_CI_AI_WS", "Chinese-Simplified-Stroke-Order-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Stroke_Order_100_CI_AI_KS", "Chinese-Simplified-Stroke-Order-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Stroke_Order_100_CI_AI_KS_WS", "Chinese-Simplified-Stroke-Order-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Stroke_Order_100_CI_AS", "Chinese-Simplified-Stroke-Order-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Stroke_Order_100_CI_AS_WS", "Chinese-Simplified-Stroke-Order-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Stroke_Order_100_CI_AS_KS", "Chinese-Simplified-Stroke-Order-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Stroke_Order_100_CI_AS_KS_WS", "Chinese-Simplified-Stroke-Order-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Stroke_Order_100_CS_AI", "Chinese-Simplified-Stroke-Order-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Stroke_Order_100_CS_AI_WS", "Chinese-Simplified-Stroke-Order-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Stroke_Order_100_CS_AI_KS", "Chinese-Simplified-Stroke-Order-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Stroke_Order_100_CS_AI_KS_WS", "Chinese-Simplified-Stroke-Order-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Stroke_Order_100_CS_AS", "Chinese-Simplified-Stroke-Order-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Stroke_Order_100_CS_AS_WS", "Chinese-Simplified-Stroke-Order-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Stroke_Order_100_CS_AS_KS", "Chinese-Simplified-Stroke-Order-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Simplified_Stroke_Order_100_CS_AS_KS_WS", "Chinese-Simplified-Stroke-Order-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_BIN2", "Chinese-Taiwan-Bopomofo, binary code point comparison sort"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_BIN", "Chinese-Taiwan-Bopomofo, binary sort"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_CI_AI", "Chinese-Taiwan-Bopomofo, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_CI_AI_WS", "Chinese-Taiwan-Bopomofo, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_CI_AI_KS", "Chinese-Taiwan-Bopomofo, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_CI_AI_KS_WS", "Chinese-Taiwan-Bopomofo, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_CI_AS", "Chinese-Taiwan-Bopomofo, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_CI_AS_WS", "Chinese-Taiwan-Bopomofo, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_CI_AS_KS", "Chinese-Taiwan-Bopomofo, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_CI_AS_KS_WS", "Chinese-Taiwan-Bopomofo, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_CS_AI", "Chinese-Taiwan-Bopomofo, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_CS_AI_WS", "Chinese-Taiwan-Bopomofo, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_CS_AI_KS", "Chinese-Taiwan-Bopomofo, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_CS_AI_KS_WS", "Chinese-Taiwan-Bopomofo, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_CS_AS", "Chinese-Taiwan-Bopomofo, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_CS_AS_WS", "Chinese-Taiwan-Bopomofo, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_CS_AS_KS", "Chinese-Taiwan-Bopomofo, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_CS_AS_KS_WS", "Chinese-Taiwan-Bopomofo, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_90_BIN2", "Chinese-Taiwan-Bopomofo-90, binary code point comparison sort"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_90_BIN", "Chinese-Taiwan-Bopomofo-90, binary sort"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_90_CI_AI", "Chinese-Taiwan-Bopomofo-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_90_CI_AI_WS", "Chinese-Taiwan-Bopomofo-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_90_CI_AI_KS", "Chinese-Taiwan-Bopomofo-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_90_CI_AI_KS_WS", "Chinese-Taiwan-Bopomofo-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_90_CI_AS", "Chinese-Taiwan-Bopomofo-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_90_CI_AS_WS", "Chinese-Taiwan-Bopomofo-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_90_CI_AS_KS", "Chinese-Taiwan-Bopomofo-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_90_CI_AS_KS_WS", "Chinese-Taiwan-Bopomofo-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_90_CS_AI", "Chinese-Taiwan-Bopomofo-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_90_CS_AI_WS", "Chinese-Taiwan-Bopomofo-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_90_CS_AI_KS", "Chinese-Taiwan-Bopomofo-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_90_CS_AI_KS_WS", "Chinese-Taiwan-Bopomofo-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_90_CS_AS", "Chinese-Taiwan-Bopomofo-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_90_CS_AS_WS", "Chinese-Taiwan-Bopomofo-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_90_CS_AS_KS", "Chinese-Taiwan-Bopomofo-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Bopomofo_90_CS_AS_KS_WS", "Chinese-Taiwan-Bopomofo-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_BIN2", "Chinese-Taiwan-Stroke, binary code point comparison sort"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_BIN", "Chinese-Taiwan-Stroke, binary sort"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_CI_AI", "Chinese-Taiwan-Stroke, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_CI_AI_WS", "Chinese-Taiwan-Stroke, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_CI_AI_KS", "Chinese-Taiwan-Stroke, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_CI_AI_KS_WS", "Chinese-Taiwan-Stroke, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_CI_AS", "Chinese-Taiwan-Stroke, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_CI_AS_WS", "Chinese-Taiwan-Stroke, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_CI_AS_KS", "Chinese-Taiwan-Stroke, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_CI_AS_KS_WS", "Chinese-Taiwan-Stroke, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_CS_AI", "Chinese-Taiwan-Stroke, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_CS_AI_WS", "Chinese-Taiwan-Stroke, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_CS_AI_KS", "Chinese-Taiwan-Stroke, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_CS_AI_KS_WS", "Chinese-Taiwan-Stroke, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_CS_AS", "Chinese-Taiwan-Stroke, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_CS_AS_WS", "Chinese-Taiwan-Stroke, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_CS_AS_KS", "Chinese-Taiwan-Stroke, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_CS_AS_KS_WS", "Chinese-Taiwan-Stroke, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_90_BIN2", "Chinese-Taiwan-Stroke-90, binary code point comparison sort"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_90_BIN", "Chinese-Taiwan-Stroke-90, binary sort"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_90_CI_AI", "Chinese-Taiwan-Stroke-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_90_CI_AI_WS", "Chinese-Taiwan-Stroke-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_90_CI_AI_KS", "Chinese-Taiwan-Stroke-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_90_CI_AI_KS_WS", "Chinese-Taiwan-Stroke-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_90_CI_AS", "Chinese-Taiwan-Stroke-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_90_CI_AS_WS", "Chinese-Taiwan-Stroke-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_90_CI_AS_KS", "Chinese-Taiwan-Stroke-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_90_CI_AS_KS_WS", "Chinese-Taiwan-Stroke-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_90_CS_AI", "Chinese-Taiwan-Stroke-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_90_CS_AI_WS", "Chinese-Taiwan-Stroke-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_90_CS_AI_KS", "Chinese-Taiwan-Stroke-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_90_CS_AI_KS_WS", "Chinese-Taiwan-Stroke-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_90_CS_AS", "Chinese-Taiwan-Stroke-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_90_CS_AS_WS", "Chinese-Taiwan-Stroke-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_90_CS_AS_KS", "Chinese-Taiwan-Stroke-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Taiwan_Stroke_90_CS_AS_KS_WS", "Chinese-Taiwan-Stroke-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Bopomofo_100_BIN2", "Chinese-Traditional-Bopomofo-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Chinese_Traditional_Bopomofo_100_BIN", "Chinese-Traditional-Bopomofo-100, binary sort"));
                _collationList.Add(new Collation("Chinese_Traditional_Bopomofo_100_CI_AI", "Chinese-Traditional-Bopomofo-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Bopomofo_100_CI_AI_WS", "Chinese-Traditional-Bopomofo-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Bopomofo_100_CI_AI_KS", "Chinese-Traditional-Bopomofo-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Bopomofo_100_CI_AI_KS_WS", "Chinese-Traditional-Bopomofo-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Bopomofo_100_CI_AS", "Chinese-Traditional-Bopomofo-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Bopomofo_100_CI_AS_WS", "Chinese-Traditional-Bopomofo-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Bopomofo_100_CI_AS_KS", "Chinese-Traditional-Bopomofo-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Bopomofo_100_CI_AS_KS_WS", "Chinese-Traditional-Bopomofo-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Bopomofo_100_CS_AI", "Chinese-Traditional-Bopomofo-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Bopomofo_100_CS_AI_WS", "Chinese-Traditional-Bopomofo-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Bopomofo_100_CS_AI_KS", "Chinese-Traditional-Bopomofo-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Bopomofo_100_CS_AI_KS_WS", "Chinese-Traditional-Bopomofo-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Bopomofo_100_CS_AS", "Chinese-Traditional-Bopomofo-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Bopomofo_100_CS_AS_WS", "Chinese-Traditional-Bopomofo-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Bopomofo_100_CS_AS_KS", "Chinese-Traditional-Bopomofo-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Bopomofo_100_CS_AS_KS_WS", "Chinese-Traditional-Bopomofo-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Pinyin_100_BIN2", "Chinese-Traditional-Pinyin-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Chinese_Traditional_Pinyin_100_BIN", "Chinese-Traditional-Pinyin-100, binary sort"));
                _collationList.Add(new Collation("Chinese_Traditional_Pinyin_100_CI_AI", "Chinese-Traditional-Pinyin-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Pinyin_100_CI_AI_WS", "Chinese-Traditional-Pinyin-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Pinyin_100_CI_AI_KS", "Chinese-Traditional-Pinyin-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Pinyin_100_CI_AI_KS_WS", "Chinese-Traditional-Pinyin-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Pinyin_100_CI_AS", "Chinese-Traditional-Pinyin-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Pinyin_100_CI_AS_WS", "Chinese-Traditional-Pinyin-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Pinyin_100_CI_AS_KS", "Chinese-Traditional-Pinyin-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Pinyin_100_CI_AS_KS_WS", "Chinese-Traditional-Pinyin-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Pinyin_100_CS_AI", "Chinese-Traditional-Pinyin-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Pinyin_100_CS_AI_WS", "Chinese-Traditional-Pinyin-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Pinyin_100_CS_AI_KS", "Chinese-Traditional-Pinyin-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Pinyin_100_CS_AI_KS_WS", "Chinese-Traditional-Pinyin-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Pinyin_100_CS_AS", "Chinese-Traditional-Pinyin-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Pinyin_100_CS_AS_WS", "Chinese-Traditional-Pinyin-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Pinyin_100_CS_AS_KS", "Chinese-Traditional-Pinyin-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Pinyin_100_CS_AS_KS_WS", "Chinese-Traditional-Pinyin-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Count_100_BIN2", "Chinese-Traditional-Stroke-Count-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Count_100_BIN", "Chinese-Traditional-Stroke-Count-100, binary sort"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Count_100_CI_AI", "Chinese-Traditional-Stroke-Count-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Count_100_CI_AI_WS", "Chinese-Traditional-Stroke-Count-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Count_100_CI_AI_KS", "Chinese-Traditional-Stroke-Count-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Count_100_CI_AI_KS_WS", "Chinese-Traditional-Stroke-Count-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Count_100_CI_AS", "Chinese-Traditional-Stroke-Count-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Count_100_CI_AS_WS", "Chinese-Traditional-Stroke-Count-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Count_100_CI_AS_KS", "Chinese-Traditional-Stroke-Count-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Count_100_CI_AS_KS_WS", "Chinese-Traditional-Stroke-Count-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Count_100_CS_AI", "Chinese-Traditional-Stroke-Count-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Count_100_CS_AI_WS", "Chinese-Traditional-Stroke-Count-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Count_100_CS_AI_KS", "Chinese-Traditional-Stroke-Count-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Count_100_CS_AI_KS_WS", "Chinese-Traditional-Stroke-Count-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Count_100_CS_AS", "Chinese-Traditional-Stroke-Count-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Count_100_CS_AS_WS", "Chinese-Traditional-Stroke-Count-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Count_100_CS_AS_KS", "Chinese-Traditional-Stroke-Count-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Count_100_CS_AS_KS_WS", "Chinese-Traditional-Stroke-Count-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Order_100_BIN2", "Chinese-Traditional-Stroke-Order-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Order_100_BIN", "Chinese-Traditional-Stroke-Order-100, binary sort"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Order_100_CI_AI", "Chinese-Traditional-Stroke-Order-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Order_100_CI_AI_WS", "Chinese-Traditional-Stroke-Order-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Order_100_CI_AI_KS", "Chinese-Traditional-Stroke-Order-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Order_100_CI_AI_KS_WS", "Chinese-Traditional-Stroke-Order-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Order_100_CI_AS", "Chinese-Traditional-Stroke-Order-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Order_100_CI_AS_WS", "Chinese-Traditional-Stroke-Order-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Order_100_CI_AS_KS", "Chinese-Traditional-Stroke-Order-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Order_100_CI_AS_KS_WS", "Chinese-Traditional-Stroke-Order-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Order_100_CS_AI", "Chinese-Traditional-Stroke-Order-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Order_100_CS_AI_WS", "Chinese-Traditional-Stroke-Order-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Order_100_CS_AI_KS", "Chinese-Traditional-Stroke-Order-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Order_100_CS_AI_KS_WS", "Chinese-Traditional-Stroke-Order-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Order_100_CS_AS", "Chinese-Traditional-Stroke-Order-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Order_100_CS_AS_WS", "Chinese-Traditional-Stroke-Order-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Order_100_CS_AS_KS", "Chinese-Traditional-Stroke-Order-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Chinese_Traditional_Stroke_Order_100_CS_AS_KS_WS", "Chinese-Traditional-Stroke-Order-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Corsican_100_BIN2", "Corsican-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Corsican_100_BIN", "Corsican-100, binary sort"));
                _collationList.Add(new Collation("Corsican_100_CI_AI", "Corsican-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Corsican_100_CI_AI_WS", "Corsican-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Corsican_100_CI_AI_KS", "Corsican-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Corsican_100_CI_AI_KS_WS", "Corsican-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Corsican_100_CI_AS", "Corsican-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Corsican_100_CI_AS_WS", "Corsican-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Corsican_100_CI_AS_KS", "Corsican-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Corsican_100_CI_AS_KS_WS", "Corsican-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Corsican_100_CS_AI", "Corsican-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Corsican_100_CS_AI_WS", "Corsican-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Corsican_100_CS_AI_KS", "Corsican-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Corsican_100_CS_AI_KS_WS", "Corsican-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Corsican_100_CS_AS", "Corsican-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Corsican_100_CS_AS_WS", "Corsican-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Corsican_100_CS_AS_KS", "Corsican-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Corsican_100_CS_AS_KS_WS", "Corsican-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Croatian_BIN2", "Croatian, binary code point comparison sort"));
                _collationList.Add(new Collation("Croatian_BIN", "Croatian, binary sort"));
                _collationList.Add(new Collation("Croatian_CI_AI", "Croatian, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Croatian_CI_AI_WS", "Croatian, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Croatian_CI_AI_KS", "Croatian, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Croatian_CI_AI_KS_WS", "Croatian, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Croatian_CI_AS", "Croatian, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Croatian_CP1250_CI_AS", "Croatian, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 92 on Code Page 1250 for non-Unicode Data"));
                _collationList.Add(new Collation("Croatian_CI_AS_WS", "Croatian, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Croatian_CI_AS_KS", "Croatian, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Croatian_CI_AS_KS_WS", "Croatian, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Croatian_CS_AI", "Croatian, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Croatian_CS_AI_WS", "Croatian, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Croatian_CS_AI_KS", "Croatian, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Croatian_CS_AI_KS_WS", "Croatian, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Croatian_CS_AS", "Croatian, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Croatian_CP1250_CS_AS", "Croatian, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 91 on Code Page 1250 for non-Unicode Data"));
                _collationList.Add(new Collation("Croatian_CS_AS_WS", "Croatian, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Croatian_CS_AS_KS", "Croatian, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Croatian_CS_AS_KS_WS", "Croatian, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Croatian_100_BIN2", "Croatian-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Croatian_100_BIN", "Croatian-100, binary sort"));
                _collationList.Add(new Collation("Croatian_100_CI_AI", "Croatian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Croatian_100_CI_AI_WS", "Croatian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Croatian_100_CI_AI_KS", "Croatian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Croatian_100_CI_AI_KS_WS", "Croatian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Croatian_100_CI_AS", "Croatian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Croatian_100_CI_AS_WS", "Croatian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Croatian_100_CI_AS_KS", "Croatian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Croatian_100_CI_AS_KS_WS", "Croatian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Croatian_100_CS_AI", "Croatian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Croatian_100_CS_AI_WS", "Croatian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Croatian_100_CS_AI_KS", "Croatian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Croatian_100_CS_AI_KS_WS", "Croatian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Croatian_100_CS_AS", "Croatian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Croatian_100_CS_AS_WS", "Croatian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Croatian_100_CS_AS_KS", "Croatian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Croatian_100_CS_AS_KS_WS", "Croatian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Cyrillic_General_BIN2", "Cyrillic-General, binary code point comparison sort"));
                _collationList.Add(new Collation("Cyrillic_General_BIN", "Cyrillic-General, binary sort"));
                _collationList.Add(new Collation("Cyrillic_General_CI_AI", "Cyrillic-General, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Cyrillic_General_CI_AI_WS", "Cyrillic-General, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Cyrillic_General_CI_AI_KS", "Cyrillic-General, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Cyrillic_General_CI_AI_KS_WS", "Cyrillic-General, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Cyrillic_General_CI_AS", "Cyrillic-General, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Cyrillic_General_CI_AS_WS", "Cyrillic-General, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Cyrillic_General_CI_AS_KS", "Cyrillic-General, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Cyrillic_General_CI_AS_KS_WS", "Cyrillic-General, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Cyrillic_General_CS_AI", "Cyrillic-General, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Cyrillic_General_CS_AI_WS", "Cyrillic-General, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Cyrillic_General_CS_AI_KS", "Cyrillic-General, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Cyrillic_General_CS_AI_KS_WS", "Cyrillic-General, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Cyrillic_General_CS_AS", "Cyrillic-General, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Cyrillic_General_CS_AS_WS", "Cyrillic-General, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Cyrillic_General_CS_AS_KS", "Cyrillic-General, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Cyrillic_General_CS_AS_KS_WS", "Cyrillic-General, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Cyrillic_General_100_BIN2", "Cyrillic-General-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Cyrillic_General_100_BIN", "Cyrillic-General-100, binary sort"));
                _collationList.Add(new Collation("Cyrillic_General_100_CI_AI", "Cyrillic-General-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Cyrillic_General_100_CI_AI_WS", "Cyrillic-General-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Cyrillic_General_100_CI_AI_KS", "Cyrillic-General-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Cyrillic_General_100_CI_AI_KS_WS", "Cyrillic-General-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Cyrillic_General_100_CI_AS", "Cyrillic-General-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Cyrillic_General_100_CI_AS_WS", "Cyrillic-General-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Cyrillic_General_100_CI_AS_KS", "Cyrillic-General-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Cyrillic_General_100_CI_AS_KS_WS", "Cyrillic-General-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Cyrillic_General_100_CS_AI", "Cyrillic-General-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Cyrillic_General_100_CS_AI_WS", "Cyrillic-General-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Cyrillic_General_100_CS_AI_KS", "Cyrillic-General-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Cyrillic_General_100_CS_AI_KS_WS", "Cyrillic-General-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Cyrillic_General_100_CS_AS", "Cyrillic-General-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Cyrillic_General_100_CS_AS_WS", "Cyrillic-General-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Cyrillic_General_100_CS_AS_KS", "Cyrillic-General-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Cyrillic_General_100_CS_AS_KS_WS", "Cyrillic-General-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Czech_BIN2", "Czech, binary code point comparison sort"));
                _collationList.Add(new Collation("Czech_BIN", "Czech, binary sort"));
                _collationList.Add(new Collation("Czech_CI_AI", "Czech, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Czech_CI_AI_WS", "Czech, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Czech_CI_AI_KS", "Czech, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Czech_CI_AI_KS_WS", "Czech, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Czech_CI_AS", "Czech, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Czech_CP1250_CI_AS", "Czech, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 84 on Code Page 1250 for non-Unicode Data"));
                _collationList.Add(new Collation("Czech_CI_AS_WS", "Czech, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Czech_CI_AS_KS", "Czech, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Czech_CI_AS_KS_WS", "Czech, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Czech_CS_AI", "Czech, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Czech_CS_AI_WS", "Czech, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Czech_CS_AI_KS", "Czech, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Czech_CS_AI_KS_WS", "Czech, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Czech_CS_AS", "Czech, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Czech_CP1250_CS_AS", "Czech, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 83 on Code Page 1250 for non-Unicode Data"));
                _collationList.Add(new Collation("Czech_CS_AS_WS", "Czech, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Czech_CS_AS_KS", "Czech, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Czech_CS_AS_KS_WS", "Czech, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Czech_100_BIN2", "Czech-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Czech_100_BIN", "Czech-100, binary sort"));
                _collationList.Add(new Collation("Czech_100_CI_AI", "Czech-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Czech_100_CI_AI_WS", "Czech-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Czech_100_CI_AI_KS", "Czech-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Czech_100_CI_AI_KS_WS", "Czech-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Czech_100_CI_AS", "Czech-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Czech_100_CI_AS_WS", "Czech-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Czech_100_CI_AS_KS", "Czech-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Czech_100_CI_AS_KS_WS", "Czech-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Czech_100_CS_AI", "Czech-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Czech_100_CS_AI_WS", "Czech-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Czech_100_CS_AI_KS", "Czech-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Czech_100_CS_AI_KS_WS", "Czech-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Czech_100_CS_AS", "Czech-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Czech_100_CS_AS_WS", "Czech-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Czech_100_CS_AS_KS", "Czech-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Czech_100_CS_AS_KS_WS", "Czech-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Danish_Greenlandic_100_BIN2", "Danish-Greenlandic-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Danish_Greenlandic_100_BIN", "Danish-Greenlandic-100, binary sort"));
                _collationList.Add(new Collation("Danish_Greenlandic_100_CI_AI", "Danish-Greenlandic-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Danish_Greenlandic_100_CI_AI_WS", "Danish-Greenlandic-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Danish_Greenlandic_100_CI_AI_KS", "Danish-Greenlandic-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Danish_Greenlandic_100_CI_AI_KS_WS", "Danish-Greenlandic-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Danish_Greenlandic_100_CI_AS", "Danish-Greenlandic-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Danish_Greenlandic_100_CI_AS_WS", "Danish-Greenlandic-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Danish_Greenlandic_100_CI_AS_KS", "Danish-Greenlandic-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Danish_Greenlandic_100_CI_AS_KS_WS", "Danish-Greenlandic-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Danish_Greenlandic_100_CS_AI", "Danish-Greenlandic-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Danish_Greenlandic_100_CS_AI_WS", "Danish-Greenlandic-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Danish_Greenlandic_100_CS_AI_KS", "Danish-Greenlandic-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Danish_Greenlandic_100_CS_AI_KS_WS", "Danish-Greenlandic-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Danish_Greenlandic_100_CS_AS", "Danish-Greenlandic-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Danish_Greenlandic_100_CS_AS_WS", "Danish-Greenlandic-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Danish_Greenlandic_100_CS_AS_KS", "Danish-Greenlandic-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Danish_Greenlandic_100_CS_AS_KS_WS", "Danish-Greenlandic-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Danish_Norwegian_BIN2", "Danish-Norwegian, binary code point comparison sort"));
                _collationList.Add(new Collation("Danish_Norwegian_BIN", "Danish-Norwegian, binary sort"));
                _collationList.Add(new Collation("Danish_Norwegian_CI_AI", "Danish-Norwegian, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Danish_Norwegian_CI_AI_WS", "Danish-Norwegian, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Danish_Norwegian_CI_AI_KS", "Danish-Norwegian, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Danish_Norwegian_CI_AI_KS_WS", "Danish-Norwegian, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Danish_Norwegian_CI_AS", "Danish-Norwegian, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Danish_Pref_CP1_CI_AS", "Danish-Norwegian, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 183 on Code Page 1252 for non-Unicode Data"));
                _collationList.Add(new Collation("Danish_Norwegian_CI_AS_WS", "Danish-Norwegian, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Danish_Norwegian_CI_AS_KS", "Danish-Norwegian, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Danish_Norwegian_CI_AS_KS_WS", "Danish-Norwegian, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Danish_Norwegian_CS_AI", "Danish-Norwegian, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Danish_Norwegian_CS_AI_WS", "Danish-Norwegian, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Danish_Norwegian_CS_AI_KS", "Danish-Norwegian, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Danish_Norwegian_CS_AI_KS_WS", "Danish-Norwegian, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Danish_Norwegian_CS_AS", "Danish-Norwegian, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_EBCDIC277_CP1_CS_AS", "Danish-Norwegian, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 212 on Code Page 1252 for non-Unicode Data"));
                _collationList.Add(new Collation("Danish_Norwegian_CS_AS_WS", "Danish-Norwegian, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Danish_Norwegian_CS_AS_KS", "Danish-Norwegian, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Danish_Norwegian_CS_AS_KS_WS", "Danish-Norwegian, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Dari_100_BIN2", "Dari-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Dari_100_BIN", "Dari-100, binary sort"));
                _collationList.Add(new Collation("Dari_100_CI_AI", "Dari-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Dari_100_CI_AI_WS", "Dari-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Dari_100_CI_AI_KS", "Dari-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Dari_100_CI_AI_KS_WS", "Dari-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Dari_100_CI_AS", "Dari-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Dari_100_CI_AS_WS", "Dari-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Dari_100_CI_AS_KS", "Dari-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Dari_100_CI_AS_KS_WS", "Dari-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Dari_100_CS_AI", "Dari-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Dari_100_CS_AI_WS", "Dari-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Dari_100_CS_AI_KS", "Dari-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Dari_100_CS_AI_KS_WS", "Dari-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Dari_100_CS_AS", "Dari-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Dari_100_CS_AS_WS", "Dari-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Dari_100_CS_AS_KS", "Dari-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Dari_100_CS_AS_KS_WS", "Dari-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Divehi_100_BIN2", "Divehi-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Divehi_100_BIN", "Divehi-100, binary sort"));
                _collationList.Add(new Collation("Divehi_100_CI_AI", "Divehi-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Divehi_100_CI_AI_WS", "Divehi-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Divehi_100_CI_AI_KS", "Divehi-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Divehi_100_CI_AI_KS_WS", "Divehi-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Divehi_100_CI_AS", "Divehi-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Divehi_100_CI_AS_WS", "Divehi-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Divehi_100_CI_AS_KS", "Divehi-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Divehi_100_CI_AS_KS_WS", "Divehi-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Divehi_100_CS_AI", "Divehi-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Divehi_100_CS_AI_WS", "Divehi-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Divehi_100_CS_AI_KS", "Divehi-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Divehi_100_CS_AI_KS_WS", "Divehi-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Divehi_100_CS_AS", "Divehi-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Divehi_100_CS_AS_WS", "Divehi-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Divehi_100_CS_AS_KS", "Divehi-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Divehi_100_CS_AS_KS_WS", "Divehi-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Divehi_90_BIN2", "Divehi-90, binary code point comparison sort"));
                _collationList.Add(new Collation("Divehi_90_BIN", "Divehi-90, binary sort"));
                _collationList.Add(new Collation("Divehi_90_CI_AI", "Divehi-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Divehi_90_CI_AI_WS", "Divehi-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Divehi_90_CI_AI_KS", "Divehi-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Divehi_90_CI_AI_KS_WS", "Divehi-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Divehi_90_CI_AS", "Divehi-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Divehi_90_CI_AS_WS", "Divehi-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Divehi_90_CI_AS_KS", "Divehi-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Divehi_90_CI_AS_KS_WS", "Divehi-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Divehi_90_CS_AI", "Divehi-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Divehi_90_CS_AI_WS", "Divehi-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Divehi_90_CS_AI_KS", "Divehi-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Divehi_90_CS_AI_KS_WS", "Divehi-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Divehi_90_CS_AS", "Divehi-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Divehi_90_CS_AS_WS", "Divehi-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Divehi_90_CS_AS_KS", "Divehi-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Divehi_90_CS_AS_KS_WS", "Divehi-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Estonian_BIN2", "Estonian, binary code point comparison sort"));
                _collationList.Add(new Collation("Estonian_BIN", "Estonian, binary sort"));
                _collationList.Add(new Collation("Estonian_CI_AI", "Estonian, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Estonian_CI_AI_WS", "Estonian, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Estonian_CI_AI_KS", "Estonian, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Estonian_CI_AI_KS_WS", "Estonian, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Estonian_CI_AS", "Estonian, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Estonian_CP1257_CI_AS", "Estonian, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 156 on Code Page 1257 for non-Unicode Data"));
                _collationList.Add(new Collation("Estonian_CI_AS_WS", "Estonian, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Estonian_CI_AS_KS", "Estonian, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Estonian_CI_AS_KS_WS", "Estonian, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Estonian_CS_AI", "Estonian, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Estonian_CS_AI_WS", "Estonian, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Estonian_CS_AI_KS", "Estonian, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Estonian_CS_AI_KS_WS", "Estonian, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Estonian_CS_AS", "Estonian, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Estonian_CP1257_CS_AS", "Estonian, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 155 on Code Page 1257 for non-Unicode Data"));
                _collationList.Add(new Collation("Estonian_CS_AS_WS", "Estonian, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Estonian_CS_AS_KS", "Estonian, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Estonian_CS_AS_KS_WS", "Estonian, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Estonian_100_BIN2", "Estonian-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Estonian_100_BIN", "Estonian-100, binary sort"));
                _collationList.Add(new Collation("Estonian_100_CI_AI", "Estonian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Estonian_100_CI_AI_WS", "Estonian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Estonian_100_CI_AI_KS", "Estonian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Estonian_100_CI_AI_KS_WS", "Estonian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Estonian_100_CI_AS", "Estonian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Estonian_100_CI_AS_WS", "Estonian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Estonian_100_CI_AS_KS", "Estonian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Estonian_100_CI_AS_KS_WS", "Estonian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Estonian_100_CS_AI", "Estonian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Estonian_100_CS_AI_WS", "Estonian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Estonian_100_CS_AI_KS", "Estonian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Estonian_100_CS_AI_KS_WS", "Estonian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Estonian_100_CS_AS", "Estonian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Estonian_100_CS_AS_WS", "Estonian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Estonian_100_CS_AS_KS", "Estonian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Estonian_100_CS_AS_KS_WS", "Estonian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_BIN2", "Finnish-Swedish, binary code point comparison sort"));
                _collationList.Add(new Collation("Finnish_Swedish_BIN", "Finnish-Swedish, binary sort"));
                _collationList.Add(new Collation("Finnish_Swedish_CI_AI", "Finnish-Swedish, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_CI_AI_WS", "Finnish-Swedish, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_CI_AI_KS", "Finnish-Swedish, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_CI_AI_KS_WS", "Finnish-Swedish, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_CI_AS", "Finnish-Swedish, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_SwedishPhone_Pref_CP1_CI_AS", "Finnish-Swedish, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 184 on Code Page 1252 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_SwedishStd_Pref_CP1_CI_AS", "Finnish-Swedish, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 185 on Code Page 1252 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Scandinavian_Pref_CP850_CI_AS", "Finnish-Swedish, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 58 on Code Page 850 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Scandinavian_CP850_CI_AS", "Finnish-Swedish, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 60 on Code Page 850 for non-Unicode Data"));
                _collationList.Add(new Collation("Finnish_Swedish_CI_AS_WS", "Finnish-Swedish, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_CI_AS_KS", "Finnish-Swedish, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_CI_AS_KS_WS", "Finnish-Swedish, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_CS_AI", "Finnish-Swedish, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_CS_AI_WS", "Finnish-Swedish, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_CS_AI_KS", "Finnish-Swedish, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_CS_AI_KS_WS", "Finnish-Swedish, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_CS_AS", "Finnish-Swedish, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_EBCDIC278_CP1_CS_AS", "Finnish-Swedish, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 213 on Code Page 1252 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Scandinavian_CP850_CS_AS", "Finnish-Swedish, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 59 on Code Page 850 for non-Unicode Data"));
                _collationList.Add(new Collation("Finnish_Swedish_CS_AS_WS", "Finnish-Swedish, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_CS_AS_KS", "Finnish-Swedish, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_CS_AS_KS_WS", "Finnish-Swedish, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_100_BIN2", "Finnish-Swedish-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Finnish_Swedish_100_BIN", "Finnish-Swedish-100, binary sort"));
                _collationList.Add(new Collation("Finnish_Swedish_100_CI_AI", "Finnish-Swedish-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_100_CI_AI_WS", "Finnish-Swedish-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_100_CI_AI_KS", "Finnish-Swedish-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_100_CI_AI_KS_WS", "Finnish-Swedish-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_100_CI_AS", "Finnish-Swedish-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_100_CI_AS_WS", "Finnish-Swedish-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_100_CI_AS_KS", "Finnish-Swedish-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_100_CI_AS_KS_WS", "Finnish-Swedish-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_100_CS_AI", "Finnish-Swedish-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_100_CS_AI_WS", "Finnish-Swedish-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_100_CS_AI_KS", "Finnish-Swedish-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_100_CS_AI_KS_WS", "Finnish-Swedish-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_100_CS_AS", "Finnish-Swedish-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_100_CS_AS_WS", "Finnish-Swedish-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_100_CS_AS_KS", "Finnish-Swedish-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Finnish_Swedish_100_CS_AS_KS_WS", "Finnish-Swedish-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("French_BIN2", "French, binary code point comparison sort"));
                _collationList.Add(new Collation("French_BIN", "French, binary sort"));
                _collationList.Add(new Collation("French_CI_AI", "French, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("French_CI_AI_WS", "French, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("French_CI_AI_KS", "French, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("French_CI_AI_KS_WS", "French, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("French_CI_AS", "French, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("French_CI_AS_WS", "French, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("French_CI_AS_KS", "French, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("French_CI_AS_KS_WS", "French, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("French_CS_AI", "French, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("French_CS_AI_WS", "French, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("French_CS_AI_KS", "French, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("French_CS_AI_KS_WS", "French, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("French_CS_AS", "French, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_EBCDIC297_CP1_CS_AS", "French, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 217 on Code Page 1252 for non-Unicode Data"));
                _collationList.Add(new Collation("French_CS_AS_WS", "French, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("French_CS_AS_KS", "French, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("French_CS_AS_KS_WS", "French, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("French_100_BIN2", "French-100, binary code point comparison sort"));
                _collationList.Add(new Collation("French_100_BIN", "French-100, binary sort"));
                _collationList.Add(new Collation("French_100_CI_AI", "French-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("French_100_CI_AI_WS", "French-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("French_100_CI_AI_KS", "French-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("French_100_CI_AI_KS_WS", "French-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("French_100_CI_AS", "French-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("French_100_CI_AS_WS", "French-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("French_100_CI_AS_KS", "French-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("French_100_CI_AS_KS_WS", "French-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("French_100_CS_AI", "French-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("French_100_CS_AI_WS", "French-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("French_100_CS_AI_KS", "French-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("French_100_CS_AI_KS_WS", "French-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("French_100_CS_AS", "French-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("French_100_CS_AS_WS", "French-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("French_100_CS_AS_KS", "French-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("French_100_CS_AS_KS_WS", "French-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Frisian_100_BIN2", "Frisian-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Frisian_100_BIN", "Frisian-100, binary sort"));
                _collationList.Add(new Collation("Frisian_100_CI_AI", "Frisian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Frisian_100_CI_AI_WS", "Frisian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Frisian_100_CI_AI_KS", "Frisian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Frisian_100_CI_AI_KS_WS", "Frisian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Frisian_100_CI_AS", "Frisian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Frisian_100_CI_AS_WS", "Frisian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Frisian_100_CI_AS_KS", "Frisian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Frisian_100_CI_AS_KS_WS", "Frisian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Frisian_100_CS_AI", "Frisian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Frisian_100_CS_AI_WS", "Frisian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Frisian_100_CS_AI_KS", "Frisian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Frisian_100_CS_AI_KS_WS", "Frisian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Frisian_100_CS_AS", "Frisian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Frisian_100_CS_AS_WS", "Frisian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Frisian_100_CS_AS_KS", "Frisian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Frisian_100_CS_AS_KS_WS", "Frisian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_BIN2", "Georgian-Modern-Sort, binary code point comparison sort"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_BIN", "Georgian-Modern-Sort, binary sort"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_CI_AI", "Georgian-Modern-Sort, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_CI_AI_WS", "Georgian-Modern-Sort, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_CI_AI_KS", "Georgian-Modern-Sort, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_CI_AI_KS_WS", "Georgian-Modern-Sort, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_CI_AS", "Georgian-Modern-Sort, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_CI_AS_WS", "Georgian-Modern-Sort, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_CI_AS_KS", "Georgian-Modern-Sort, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_CI_AS_KS_WS", "Georgian-Modern-Sort, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_CS_AI", "Georgian-Modern-Sort, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_CS_AI_WS", "Georgian-Modern-Sort, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_CS_AI_KS", "Georgian-Modern-Sort, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_CS_AI_KS_WS", "Georgian-Modern-Sort, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_CS_AS", "Georgian-Modern-Sort, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_CS_AS_WS", "Georgian-Modern-Sort, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_CS_AS_KS", "Georgian-Modern-Sort, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_CS_AS_KS_WS", "Georgian-Modern-Sort, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_100_BIN2", "Georgian-Modern-Sort-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_100_BIN", "Georgian-Modern-Sort-100, binary sort"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_100_CI_AI", "Georgian-Modern-Sort-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_100_CI_AI_WS", "Georgian-Modern-Sort-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_100_CI_AI_KS", "Georgian-Modern-Sort-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_100_CI_AI_KS_WS", "Georgian-Modern-Sort-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_100_CI_AS", "Georgian-Modern-Sort-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_100_CI_AS_WS", "Georgian-Modern-Sort-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_100_CI_AS_KS", "Georgian-Modern-Sort-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_100_CI_AS_KS_WS", "Georgian-Modern-Sort-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_100_CS_AI", "Georgian-Modern-Sort-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_100_CS_AI_WS", "Georgian-Modern-Sort-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_100_CS_AI_KS", "Georgian-Modern-Sort-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_100_CS_AI_KS_WS", "Georgian-Modern-Sort-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_100_CS_AS", "Georgian-Modern-Sort-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_100_CS_AS_WS", "Georgian-Modern-Sort-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_100_CS_AS_KS", "Georgian-Modern-Sort-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Georgian_Modern_Sort_100_CS_AS_KS_WS", "Georgian-Modern-Sort-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("German_PhoneBook_BIN2", "German-PhoneBook, binary code point comparison sort"));
                _collationList.Add(new Collation("German_PhoneBook_BIN", "German-PhoneBook, binary sort"));
                _collationList.Add(new Collation("German_PhoneBook_CI_AI", "German-PhoneBook, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("German_PhoneBook_CI_AI_WS", "German-PhoneBook, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("German_PhoneBook_CI_AI_KS", "German-PhoneBook, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("German_PhoneBook_CI_AI_KS_WS", "German-PhoneBook, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("German_PhoneBook_CI_AS", "German-PhoneBook, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("German_PhoneBook_CI_AS_WS", "German-PhoneBook, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("German_PhoneBook_CI_AS_KS", "German-PhoneBook, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("German_PhoneBook_CI_AS_KS_WS", "German-PhoneBook, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("German_PhoneBook_CS_AI", "German-PhoneBook, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("German_PhoneBook_CS_AI_WS", "German-PhoneBook, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("German_PhoneBook_CS_AI_KS", "German-PhoneBook, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("German_PhoneBook_CS_AI_KS_WS", "German-PhoneBook, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("German_PhoneBook_CS_AS", "German-PhoneBook, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_EBCDIC273_CP1_CS_AS", "German-PhoneBook, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 211 on Code Page 1252 for non-Unicode Data"));
                _collationList.Add(new Collation("German_PhoneBook_CS_AS_WS", "German-PhoneBook, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("German_PhoneBook_CS_AS_KS", "German-PhoneBook, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("German_PhoneBook_CS_AS_KS_WS", "German-PhoneBook, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("German_PhoneBook_100_BIN2", "German-PhoneBook-100, binary code point comparison sort"));
                _collationList.Add(new Collation("German_PhoneBook_100_BIN", "German-PhoneBook-100, binary sort"));
                _collationList.Add(new Collation("German_PhoneBook_100_CI_AI", "German-PhoneBook-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("German_PhoneBook_100_CI_AI_WS", "German-PhoneBook-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("German_PhoneBook_100_CI_AI_KS", "German-PhoneBook-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("German_PhoneBook_100_CI_AI_KS_WS", "German-PhoneBook-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("German_PhoneBook_100_CI_AS", "German-PhoneBook-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("German_PhoneBook_100_CI_AS_WS", "German-PhoneBook-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("German_PhoneBook_100_CI_AS_KS", "German-PhoneBook-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("German_PhoneBook_100_CI_AS_KS_WS", "German-PhoneBook-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("German_PhoneBook_100_CS_AI", "German-PhoneBook-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("German_PhoneBook_100_CS_AI_WS", "German-PhoneBook-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("German_PhoneBook_100_CS_AI_KS", "German-PhoneBook-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("German_PhoneBook_100_CS_AI_KS_WS", "German-PhoneBook-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("German_PhoneBook_100_CS_AS", "German-PhoneBook-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("German_PhoneBook_100_CS_AS_WS", "German-PhoneBook-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("German_PhoneBook_100_CS_AS_KS", "German-PhoneBook-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("German_PhoneBook_100_CS_AS_KS_WS", "German-PhoneBook-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Greek_BIN2", "Greek, binary code point comparison sort"));
                _collationList.Add(new Collation("Greek_BIN", "Greek, binary sort"));
                _collationList.Add(new Collation("Greek_CI_AI", "Greek, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Greek_CI_AI_WS", "Greek, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Greek_CI_AI_KS", "Greek, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Greek_CI_AI_KS_WS", "Greek, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Greek_CI_AS", "Greek, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Greek_CI_AS_WS", "Greek, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Greek_CI_AS_KS", "Greek, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Greek_CI_AS_KS_WS", "Greek, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Greek_CS_AI", "Greek, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Greek_CS_AI_WS", "Greek, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Greek_CS_AI_KS", "Greek, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Greek_CS_AI_KS_WS", "Greek, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Greek_CS_AS", "Greek, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Greek_CS_AS_WS", "Greek, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Greek_CS_AS_KS", "Greek, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Greek_CS_AS_KS_WS", "Greek, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Greek_100_BIN2", "Greek-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Greek_100_BIN", "Greek-100, binary sort"));
                _collationList.Add(new Collation("Greek_100_CI_AI", "Greek-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Greek_100_CI_AI_WS", "Greek-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Greek_100_CI_AI_KS", "Greek-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Greek_100_CI_AI_KS_WS", "Greek-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Greek_100_CI_AS", "Greek-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Greek_100_CI_AS_WS", "Greek-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Greek_100_CI_AS_KS", "Greek-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Greek_100_CI_AS_KS_WS", "Greek-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Greek_100_CS_AI", "Greek-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Greek_100_CS_AI_WS", "Greek-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Greek_100_CS_AI_KS", "Greek-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Greek_100_CS_AI_KS_WS", "Greek-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Greek_100_CS_AS", "Greek-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Greek_100_CS_AS_WS", "Greek-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Greek_100_CS_AS_KS", "Greek-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Greek_100_CS_AS_KS_WS", "Greek-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hebrew_BIN2", "Hebrew, binary code point comparison sort"));
                _collationList.Add(new Collation("Hebrew_BIN", "Hebrew, binary sort"));
                _collationList.Add(new Collation("Hebrew_CI_AI", "Hebrew, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hebrew_CI_AI_WS", "Hebrew, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hebrew_CI_AI_KS", "Hebrew, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hebrew_CI_AI_KS_WS", "Hebrew, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hebrew_CI_AS", "Hebrew, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hebrew_CI_AS_WS", "Hebrew, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hebrew_CI_AS_KS", "Hebrew, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hebrew_CI_AS_KS_WS", "Hebrew, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hebrew_CS_AI", "Hebrew, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hebrew_CS_AI_WS", "Hebrew, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hebrew_CS_AI_KS", "Hebrew, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hebrew_CS_AI_KS_WS", "Hebrew, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hebrew_CS_AS", "Hebrew, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hebrew_CS_AS_WS", "Hebrew, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hebrew_CS_AS_KS", "Hebrew, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hebrew_CS_AS_KS_WS", "Hebrew, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hebrew_100_BIN2", "Hebrew-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Hebrew_100_BIN", "Hebrew-100, binary sort"));
                _collationList.Add(new Collation("Hebrew_100_CI_AI", "Hebrew-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hebrew_100_CI_AI_WS", "Hebrew-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hebrew_100_CI_AI_KS", "Hebrew-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hebrew_100_CI_AI_KS_WS", "Hebrew-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hebrew_100_CI_AS", "Hebrew-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hebrew_100_CI_AS_WS", "Hebrew-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hebrew_100_CI_AS_KS", "Hebrew-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hebrew_100_CI_AS_KS_WS", "Hebrew-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hebrew_100_CS_AI", "Hebrew-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hebrew_100_CS_AI_WS", "Hebrew-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hebrew_100_CS_AI_KS", "Hebrew-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hebrew_100_CS_AI_KS_WS", "Hebrew-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hebrew_100_CS_AS", "Hebrew-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hebrew_100_CS_AS_WS", "Hebrew-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hebrew_100_CS_AS_KS", "Hebrew-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hebrew_100_CS_AS_KS_WS", "Hebrew-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_BIN2", "Hungarian, binary code point comparison sort"));
                _collationList.Add(new Collation("Hungarian_BIN", "Hungarian, binary sort"));
                _collationList.Add(new Collation("Hungarian_CI_AI", "Hungarian, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_CI_AI_WS", "Hungarian, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_CI_AI_KS", "Hungarian, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_CI_AI_KS_WS", "Hungarian, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_CI_AS", "Hungarian, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Hungarian_CP1250_CI_AS", "Hungarian, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 86 on Code Page 1250 for non-Unicode Data"));
                _collationList.Add(new Collation("Hungarian_CI_AS_WS", "Hungarian, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_CI_AS_KS", "Hungarian, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_CI_AS_KS_WS", "Hungarian, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_CS_AI", "Hungarian, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_CS_AI_WS", "Hungarian, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_CS_AI_KS", "Hungarian, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_CS_AI_KS_WS", "Hungarian, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_CS_AS", "Hungarian, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Hungarian_CP1250_CS_AS", "Hungarian, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 85 on Code Page 1250 for non-Unicode Data"));
                _collationList.Add(new Collation("Hungarian_CS_AS_WS", "Hungarian, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_CS_AS_KS", "Hungarian, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_CS_AS_KS_WS", "Hungarian, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_100_BIN2", "Hungarian-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Hungarian_100_BIN", "Hungarian-100, binary sort"));
                _collationList.Add(new Collation("Hungarian_100_CI_AI", "Hungarian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_100_CI_AI_WS", "Hungarian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_100_CI_AI_KS", "Hungarian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_100_CI_AI_KS_WS", "Hungarian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_100_CI_AS", "Hungarian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_100_CI_AS_WS", "Hungarian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_100_CI_AS_KS", "Hungarian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_100_CI_AS_KS_WS", "Hungarian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_100_CS_AI", "Hungarian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_100_CS_AI_WS", "Hungarian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_100_CS_AI_KS", "Hungarian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_100_CS_AI_KS_WS", "Hungarian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_100_CS_AS", "Hungarian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_100_CS_AS_WS", "Hungarian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_100_CS_AS_KS", "Hungarian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_100_CS_AS_KS_WS", "Hungarian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_BIN2", "Hungarian-Technical, binary code point comparison sort"));
                _collationList.Add(new Collation("Hungarian_Technical_BIN", "Hungarian-Technical, binary sort"));
                _collationList.Add(new Collation("Hungarian_Technical_CI_AI", "Hungarian-Technical, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_CI_AI_WS", "Hungarian-Technical, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_CI_AI_KS", "Hungarian-Technical, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_CI_AI_KS_WS", "Hungarian-Technical, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_CI_AS", "Hungarian-Technical, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_CI_AS_WS", "Hungarian-Technical, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_CI_AS_KS", "Hungarian-Technical, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_CI_AS_KS_WS", "Hungarian-Technical, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_CS_AI", "Hungarian-Technical, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_CS_AI_WS", "Hungarian-Technical, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_CS_AI_KS", "Hungarian-Technical, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_CS_AI_KS_WS", "Hungarian-Technical, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_CS_AS", "Hungarian-Technical, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_CS_AS_WS", "Hungarian-Technical, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_CS_AS_KS", "Hungarian-Technical, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_CS_AS_KS_WS", "Hungarian-Technical, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_100_BIN2", "Hungarian-Technical-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Hungarian_Technical_100_BIN", "Hungarian-Technical-100, binary sort"));
                _collationList.Add(new Collation("Hungarian_Technical_100_CI_AI", "Hungarian-Technical-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_100_CI_AI_WS", "Hungarian-Technical-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_100_CI_AI_KS", "Hungarian-Technical-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_100_CI_AI_KS_WS", "Hungarian-Technical-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_100_CI_AS", "Hungarian-Technical-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_100_CI_AS_WS", "Hungarian-Technical-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_100_CI_AS_KS", "Hungarian-Technical-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_100_CI_AS_KS_WS", "Hungarian-Technical-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_100_CS_AI", "Hungarian-Technical-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_100_CS_AI_WS", "Hungarian-Technical-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_100_CS_AI_KS", "Hungarian-Technical-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_100_CS_AI_KS_WS", "Hungarian-Technical-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_100_CS_AS", "Hungarian-Technical-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_100_CS_AS_WS", "Hungarian-Technical-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_100_CS_AS_KS", "Hungarian-Technical-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Hungarian_Technical_100_CS_AS_KS_WS", "Hungarian-Technical-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Icelandic_BIN2", "Icelandic, binary code point comparison sort"));
                _collationList.Add(new Collation("Icelandic_BIN", "Icelandic, binary sort"));
                _collationList.Add(new Collation("Icelandic_CI_AI", "Icelandic, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Icelandic_CI_AI_WS", "Icelandic, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Icelandic_CI_AI_KS", "Icelandic, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Icelandic_CI_AI_KS_WS", "Icelandic, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Icelandic_CI_AS", "Icelandic, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Icelandic_Pref_CP1_CI_AS", "Icelandic, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 186 on Code Page 1252 for non-Unicode Data"));
                _collationList.Add(new Collation("Icelandic_CI_AS_WS", "Icelandic, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Icelandic_CI_AS_KS", "Icelandic, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Icelandic_CI_AS_KS_WS", "Icelandic, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Icelandic_CS_AI", "Icelandic, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Icelandic_CS_AI_WS", "Icelandic, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Icelandic_CS_AI_KS", "Icelandic, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Icelandic_CS_AI_KS_WS", "Icelandic, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Icelandic_CS_AS", "Icelandic, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Icelandic_CS_AS_WS", "Icelandic, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Icelandic_CS_AS_KS", "Icelandic, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Icelandic_CS_AS_KS_WS", "Icelandic, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Icelandic_100_BIN2", "Icelandic-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Icelandic_100_BIN", "Icelandic-100, binary sort"));
                _collationList.Add(new Collation("Icelandic_100_CI_AI", "Icelandic-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Icelandic_100_CI_AI_WS", "Icelandic-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Icelandic_100_CI_AI_KS", "Icelandic-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Icelandic_100_CI_AI_KS_WS", "Icelandic-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Icelandic_100_CI_AS", "Icelandic-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Icelandic_100_CI_AS_WS", "Icelandic-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Icelandic_100_CI_AS_KS", "Icelandic-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Icelandic_100_CI_AS_KS_WS", "Icelandic-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Icelandic_100_CS_AI", "Icelandic-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Icelandic_100_CS_AI_WS", "Icelandic-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Icelandic_100_CS_AI_KS", "Icelandic-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Icelandic_100_CS_AI_KS_WS", "Icelandic-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Icelandic_100_CS_AS", "Icelandic-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Icelandic_100_CS_AS_WS", "Icelandic-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Icelandic_100_CS_AS_KS", "Icelandic-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Icelandic_100_CS_AS_KS_WS", "Icelandic-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Indic_General_100_BIN2", "Indic-General-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Indic_General_100_BIN", "Indic-General-100, binary sort"));
                _collationList.Add(new Collation("Indic_General_100_CI_AI", "Indic-General-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Indic_General_100_CI_AI_WS", "Indic-General-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Indic_General_100_CI_AI_KS", "Indic-General-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Indic_General_100_CI_AI_KS_WS", "Indic-General-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Indic_General_100_CI_AS", "Indic-General-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Indic_General_100_CI_AS_WS", "Indic-General-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Indic_General_100_CI_AS_KS", "Indic-General-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Indic_General_100_CI_AS_KS_WS", "Indic-General-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Indic_General_100_CS_AI", "Indic-General-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Indic_General_100_CS_AI_WS", "Indic-General-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Indic_General_100_CS_AI_KS", "Indic-General-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Indic_General_100_CS_AI_KS_WS", "Indic-General-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Indic_General_100_CS_AS", "Indic-General-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Indic_General_100_CS_AS_WS", "Indic-General-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Indic_General_100_CS_AS_KS", "Indic-General-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Indic_General_100_CS_AS_KS_WS", "Indic-General-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Indic_General_90_BIN2", "Indic-General-90, binary code point comparison sort"));
                _collationList.Add(new Collation("Indic_General_90_BIN", "Indic-General-90, binary sort"));
                _collationList.Add(new Collation("Indic_General_90_CI_AI", "Indic-General-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Indic_General_90_CI_AI_WS", "Indic-General-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Indic_General_90_CI_AI_KS", "Indic-General-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Indic_General_90_CI_AI_KS_WS", "Indic-General-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Indic_General_90_CI_AS", "Indic-General-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Indic_General_90_CI_AS_WS", "Indic-General-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Indic_General_90_CI_AS_KS", "Indic-General-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Indic_General_90_CI_AS_KS_WS", "Indic-General-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Indic_General_90_CS_AI", "Indic-General-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Indic_General_90_CS_AI_WS", "Indic-General-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Indic_General_90_CS_AI_KS", "Indic-General-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Indic_General_90_CS_AI_KS_WS", "Indic-General-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Indic_General_90_CS_AS", "Indic-General-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Indic_General_90_CS_AS_WS", "Indic-General-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Indic_General_90_CS_AS_KS", "Indic-General-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Indic_General_90_CS_AS_KS_WS", "Indic-General-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_BIN2", "Japanese, binary code point comparison sort"));
                _collationList.Add(new Collation("Japanese_BIN", "Japanese, binary sort"));
                _collationList.Add(new Collation("Japanese_CI_AI", "Japanese, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_CI_AI_WS", "Japanese, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_CI_AI_KS", "Japanese, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_CI_AI_KS_WS", "Japanese, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_CI_AS", "Japanese, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_CI_AS_WS", "Japanese, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_CI_AS_KS", "Japanese, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_CI_AS_KS_WS", "Japanese, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_CS_AI", "Japanese, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_CS_AI_WS", "Japanese, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_CS_AI_KS", "Japanese, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_CS_AI_KS_WS", "Japanese, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_CS_AS", "Japanese, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_CS_AS_WS", "Japanese, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_CS_AS_KS", "Japanese, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_CS_AS_KS_WS", "Japanese, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_90_BIN2", "Japanese-90, binary code point comparison sort"));
                _collationList.Add(new Collation("Japanese_90_BIN", "Japanese-90, binary sort"));
                _collationList.Add(new Collation("Japanese_90_CI_AI", "Japanese-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_90_CI_AI_WS", "Japanese-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_90_CI_AI_KS", "Japanese-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_90_CI_AI_KS_WS", "Japanese-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_90_CI_AS", "Japanese-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_90_CI_AS_WS", "Japanese-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_90_CI_AS_KS", "Japanese-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_90_CI_AS_KS_WS", "Japanese-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_90_CS_AI", "Japanese-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_90_CS_AI_WS", "Japanese-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_90_CS_AI_KS", "Japanese-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_90_CS_AI_KS_WS", "Japanese-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_90_CS_AS", "Japanese-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_90_CS_AS_WS", "Japanese-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_90_CS_AS_KS", "Japanese-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_90_CS_AS_KS_WS", "Japanese-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_Bushu_Kakusu_100_BIN2", "Japanese-Bushu-Kakusu-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Japanese_Bushu_Kakusu_100_BIN", "Japanese-Bushu-Kakusu-100, binary sort"));
                _collationList.Add(new Collation("Japanese_Bushu_Kakusu_100_CI_AI", "Japanese-Bushu-Kakusu-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_Bushu_Kakusu_100_CI_AI_WS", "Japanese-Bushu-Kakusu-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_Bushu_Kakusu_100_CI_AI_KS", "Japanese-Bushu-Kakusu-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_Bushu_Kakusu_100_CI_AI_KS_WS", "Japanese-Bushu-Kakusu-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_Bushu_Kakusu_100_CI_AS", "Japanese-Bushu-Kakusu-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_Bushu_Kakusu_100_CI_AS_WS", "Japanese-Bushu-Kakusu-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_Bushu_Kakusu_100_CI_AS_KS", "Japanese-Bushu-Kakusu-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_Bushu_Kakusu_100_CI_AS_KS_WS", "Japanese-Bushu-Kakusu-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_Bushu_Kakusu_100_CS_AI", "Japanese-Bushu-Kakusu-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_Bushu_Kakusu_100_CS_AI_WS", "Japanese-Bushu-Kakusu-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_Bushu_Kakusu_100_CS_AI_KS", "Japanese-Bushu-Kakusu-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_Bushu_Kakusu_100_CS_AI_KS_WS", "Japanese-Bushu-Kakusu-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_Bushu_Kakusu_100_CS_AS", "Japanese-Bushu-Kakusu-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_Bushu_Kakusu_100_CS_AS_WS", "Japanese-Bushu-Kakusu-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_Bushu_Kakusu_100_CS_AS_KS", "Japanese-Bushu-Kakusu-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_Bushu_Kakusu_100_CS_AS_KS_WS", "Japanese-Bushu-Kakusu-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_Unicode_BIN2", "Japanese-Unicode, binary code point comparison sort"));
                _collationList.Add(new Collation("Japanese_Unicode_BIN", "Japanese-Unicode, binary sort"));
                _collationList.Add(new Collation("Japanese_Unicode_CI_AI", "Japanese-Unicode, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_Unicode_CI_AI_WS", "Japanese-Unicode, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_Unicode_CI_AI_KS", "Japanese-Unicode, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_Unicode_CI_AI_KS_WS", "Japanese-Unicode, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_Unicode_CI_AS", "Japanese-Unicode, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_Unicode_CI_AS_WS", "Japanese-Unicode, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_Unicode_CI_AS_KS", "Japanese-Unicode, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_Unicode_CI_AS_KS_WS", "Japanese-Unicode, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_Unicode_CS_AI", "Japanese-Unicode, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_Unicode_CS_AI_WS", "Japanese-Unicode, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_Unicode_CS_AI_KS", "Japanese-Unicode, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_Unicode_CS_AI_KS_WS", "Japanese-Unicode, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_Unicode_CS_AS", "Japanese-Unicode, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_Unicode_CS_AS_WS", "Japanese-Unicode, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_Unicode_CS_AS_KS", "Japanese-Unicode, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_Unicode_CS_AS_KS_WS", "Japanese-Unicode, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_XJIS_100_BIN2", "Japanese-XJIS-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Japanese_XJIS_100_BIN", "Japanese-XJIS-100, binary sort"));
                _collationList.Add(new Collation("Japanese_XJIS_100_CI_AI", "Japanese-XJIS-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_XJIS_100_CI_AI_WS", "Japanese-XJIS-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_XJIS_100_CI_AI_KS", "Japanese-XJIS-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_XJIS_100_CI_AI_KS_WS", "Japanese-XJIS-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_XJIS_100_CI_AS", "Japanese-XJIS-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_XJIS_100_CI_AS_WS", "Japanese-XJIS-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_XJIS_100_CI_AS_KS", "Japanese-XJIS-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_XJIS_100_CI_AS_KS_WS", "Japanese-XJIS-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_XJIS_100_CS_AI", "Japanese-XJIS-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_XJIS_100_CS_AI_WS", "Japanese-XJIS-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_XJIS_100_CS_AI_KS", "Japanese-XJIS-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_XJIS_100_CS_AI_KS_WS", "Japanese-XJIS-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_XJIS_100_CS_AS", "Japanese-XJIS-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_XJIS_100_CS_AS_WS", "Japanese-XJIS-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Japanese_XJIS_100_CS_AS_KS", "Japanese-XJIS-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Japanese_XJIS_100_CS_AS_KS_WS", "Japanese-XJIS-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Kazakh_100_BIN2", "Kazakh-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Kazakh_100_BIN", "Kazakh-100, binary sort"));
                _collationList.Add(new Collation("Kazakh_100_CI_AI", "Kazakh-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Kazakh_100_CI_AI_WS", "Kazakh-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Kazakh_100_CI_AI_KS", "Kazakh-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Kazakh_100_CI_AI_KS_WS", "Kazakh-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Kazakh_100_CI_AS", "Kazakh-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Kazakh_100_CI_AS_WS", "Kazakh-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Kazakh_100_CI_AS_KS", "Kazakh-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Kazakh_100_CI_AS_KS_WS", "Kazakh-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Kazakh_100_CS_AI", "Kazakh-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Kazakh_100_CS_AI_WS", "Kazakh-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Kazakh_100_CS_AI_KS", "Kazakh-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Kazakh_100_CS_AI_KS_WS", "Kazakh-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Kazakh_100_CS_AS", "Kazakh-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Kazakh_100_CS_AS_WS", "Kazakh-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Kazakh_100_CS_AS_KS", "Kazakh-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Kazakh_100_CS_AS_KS_WS", "Kazakh-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Kazakh_90_BIN2", "Kazakh-90, binary code point comparison sort"));
                _collationList.Add(new Collation("Kazakh_90_BIN", "Kazakh-90, binary sort"));
                _collationList.Add(new Collation("Kazakh_90_CI_AI", "Kazakh-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Kazakh_90_CI_AI_WS", "Kazakh-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Kazakh_90_CI_AI_KS", "Kazakh-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Kazakh_90_CI_AI_KS_WS", "Kazakh-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Kazakh_90_CI_AS", "Kazakh-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Kazakh_90_CI_AS_WS", "Kazakh-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Kazakh_90_CI_AS_KS", "Kazakh-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Kazakh_90_CI_AS_KS_WS", "Kazakh-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Kazakh_90_CS_AI", "Kazakh-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Kazakh_90_CS_AI_WS", "Kazakh-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Kazakh_90_CS_AI_KS", "Kazakh-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Kazakh_90_CS_AI_KS_WS", "Kazakh-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Kazakh_90_CS_AS", "Kazakh-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Kazakh_90_CS_AS_WS", "Kazakh-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Kazakh_90_CS_AS_KS", "Kazakh-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Kazakh_90_CS_AS_KS_WS", "Kazakh-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Khmer_100_BIN2", "Khmer-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Khmer_100_BIN", "Khmer-100, binary sort"));
                _collationList.Add(new Collation("Khmer_100_CI_AI", "Khmer-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Khmer_100_CI_AI_WS", "Khmer-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Khmer_100_CI_AI_KS", "Khmer-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Khmer_100_CI_AI_KS_WS", "Khmer-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Khmer_100_CI_AS", "Khmer-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Khmer_100_CI_AS_WS", "Khmer-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Khmer_100_CI_AS_KS", "Khmer-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Khmer_100_CI_AS_KS_WS", "Khmer-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Khmer_100_CS_AI", "Khmer-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Khmer_100_CS_AI_WS", "Khmer-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Khmer_100_CS_AI_KS", "Khmer-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Khmer_100_CS_AI_KS_WS", "Khmer-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Khmer_100_CS_AS", "Khmer-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Khmer_100_CS_AS_WS", "Khmer-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Khmer_100_CS_AS_KS", "Khmer-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Khmer_100_CS_AS_KS_WS", "Khmer-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_100_BIN2", "Korean-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Korean_100_BIN", "Korean-100, binary sort"));
                _collationList.Add(new Collation("Korean_100_CI_AI", "Korean-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_100_CI_AI_WS", "Korean-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_100_CI_AI_KS", "Korean-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_100_CI_AI_KS_WS", "Korean-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_100_CI_AS", "Korean-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_100_CI_AS_WS", "Korean-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_100_CI_AS_KS", "Korean-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_100_CI_AS_KS_WS", "Korean-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_100_CS_AI", "Korean-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_100_CS_AI_WS", "Korean-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_100_CS_AI_KS", "Korean-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_100_CS_AI_KS_WS", "Korean-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_100_CS_AS", "Korean-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_100_CS_AS_WS", "Korean-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_100_CS_AS_KS", "Korean-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_100_CS_AS_KS_WS", "Korean-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_90_BIN2", "Korean-90, binary code point comparison sort"));
                _collationList.Add(new Collation("Korean_90_BIN", "Korean-90, binary sort"));
                _collationList.Add(new Collation("Korean_90_CI_AI", "Korean-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_90_CI_AI_WS", "Korean-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_90_CI_AI_KS", "Korean-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_90_CI_AI_KS_WS", "Korean-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_90_CI_AS", "Korean-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_90_CI_AS_WS", "Korean-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_90_CI_AS_KS", "Korean-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_90_CI_AS_KS_WS", "Korean-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_90_CS_AI", "Korean-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_90_CS_AI_WS", "Korean-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_90_CS_AI_KS", "Korean-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_90_CS_AI_KS_WS", "Korean-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_90_CS_AS", "Korean-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_90_CS_AS_WS", "Korean-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_90_CS_AS_KS", "Korean-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_90_CS_AS_KS_WS", "Korean-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_Wansung_BIN2", "Korean-Wansung, binary code point comparison sort"));
                _collationList.Add(new Collation("Korean_Wansung_BIN", "Korean-Wansung, binary sort"));
                _collationList.Add(new Collation("Korean_Wansung_CI_AI", "Korean-Wansung, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_Wansung_CI_AI_WS", "Korean-Wansung, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_Wansung_CI_AI_KS", "Korean-Wansung, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_Wansung_CI_AI_KS_WS", "Korean-Wansung, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_Wansung_CI_AS", "Korean-Wansung, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_Wansung_CI_AS_WS", "Korean-Wansung, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_Wansung_CI_AS_KS", "Korean-Wansung, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_Wansung_CI_AS_KS_WS", "Korean-Wansung, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_Wansung_CS_AI", "Korean-Wansung, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_Wansung_CS_AI_WS", "Korean-Wansung, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_Wansung_CS_AI_KS", "Korean-Wansung, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_Wansung_CS_AI_KS_WS", "Korean-Wansung, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_Wansung_CS_AS", "Korean-Wansung, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_Wansung_CS_AS_WS", "Korean-Wansung, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Korean_Wansung_CS_AS_KS", "Korean-Wansung, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Korean_Wansung_CS_AS_KS_WS", "Korean-Wansung, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Lao_100_BIN2", "Lao-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Lao_100_BIN", "Lao-100, binary sort"));
                _collationList.Add(new Collation("Lao_100_CI_AI", "Lao-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Lao_100_CI_AI_WS", "Lao-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Lao_100_CI_AI_KS", "Lao-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Lao_100_CI_AI_KS_WS", "Lao-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Lao_100_CI_AS", "Lao-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Lao_100_CI_AS_WS", "Lao-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Lao_100_CI_AS_KS", "Lao-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Lao_100_CI_AS_KS_WS", "Lao-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Lao_100_CS_AI", "Lao-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Lao_100_CS_AI_WS", "Lao-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Lao_100_CS_AI_KS", "Lao-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Lao_100_CS_AI_KS_WS", "Lao-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Lao_100_CS_AS", "Lao-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Lao_100_CS_AS_WS", "Lao-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Lao_100_CS_AS_KS", "Lao-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Lao_100_CS_AS_KS_WS", "Lao-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Latin1_General_BIN2", "Latin1-General, binary code point comparison sort"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP437_BIN2", "Latin1-General, binary code point comparison sort for Unicode Data, SQL Server Sort Order 30 on Code Page 437 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP850_BIN2", "Latin1-General, binary code point comparison sort for Unicode Data, SQL Server Sort Order 40 on Code Page 850 for non-Unicode Data"));
                _collationList.Add(new Collation("Latin1_General_BIN", "Latin1-General, binary sort"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP437_BIN", "Latin1-General, binary sort for Unicode Data, SQL Server Sort Order 30 on Code Page 437 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP850_BIN", "Latin1-General, binary sort for Unicode Data, SQL Server Sort Order 40 on Code Page 850 for non-Unicode Data"));
                _collationList.Add(new Collation("Latin1_General_CI_AI", "Latin1-General, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP1253_CI_AI", "Latin1-General, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 124 on Code Page 1253 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP437_CI_AI", "Latin1-General, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 34 on Code Page 437 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP850_CI_AI", "Latin1-General, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 44 on Code Page 850 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP1_CI_AI", "Latin1-General, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 54 on Code Page 1252 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_AltDiction_CP850_CI_AI", "Latin1-General, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 57 on Code Page 850 for non-Unicode Data"));
                _collationList.Add(new Collation("Latin1_General_CI_AI_WS", "Latin1-General, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Latin1_General_CI_AI_KS", "Latin1-General, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Latin1_General_CI_AI_KS_WS", "Latin1-General, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Latin1_General_CI_AS", "Latin1-General, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP1251_CI_AS", "Latin1-General, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 106 on Code Page 1251 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP1253_CI_AS", "Latin1-General, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 114 on Code Page 1253 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP1255_CI_AS", "Latin1-General, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 138 on Code Page 1255 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP1256_CI_AS", "Latin1-General, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 146 on Code Page 1256 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP1257_CI_AS", "Latin1-General, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 154 on Code Page 1257 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP437_CI_AS", "Latin1-General, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 32 on Code Page 437 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_Pref_CP437_CI_AS", "Latin1-General, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 33 on Code Page 437 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP850_CI_AS", "Latin1-General, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 42 on Code Page 850 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_Pref_CP850_CI_AS", "Latin1-General, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 43 on Code Page 850 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_1xCompat_CP850_CI_AS", "Latin1-General, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 49 on Code Page 850 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP1_CI_AS", "Latin1-General, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 52 on Code Page 1252 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_Pref_CP1_CI_AS", "Latin1-General, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 53 on Code Page 1252 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_AltDiction_Pref_CP850_CI_AS", "Latin1-General, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 56 on Code Page 850 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_AltDiction_CP850_CI_AS", "Latin1-General, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 61 on Code Page 850 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP1250_CI_AS", "Latin1-General, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 82 on Code Page 1250 for non-Unicode Data"));
                _collationList.Add(new Collation("Latin1_General_CI_AS_WS", "Latin1-General, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Latin1_General_CI_AS_KS", "Latin1-General, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Latin1_General_CI_AS_KS_WS", "Latin1-General, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Latin1_General_CS_AI", "Latin1-General, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Latin1_General_CS_AI_WS", "Latin1-General, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Latin1_General_CS_AI_KS", "Latin1-General, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Latin1_General_CS_AI_KS_WS", "Latin1-General, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Latin1_General_CS_AS", "Latin1-General, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP1251_CS_AS", "Latin1-General, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 105 on Code Page 1251 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP1253_CS_AS", "Latin1-General, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 113 on Code Page 1253 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_MixDiction_CP1253_CS_AS", "Latin1-General, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 120 on Code Page 1253 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_AltDiction2_CP1253_CS_AS", "Latin1-General, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 122 on Code Page 1253 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP1255_CS_AS", "Latin1-General, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 137 on Code Page 1255 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP1256_CS_AS", "Latin1-General, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 145 on Code Page 1256 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP1257_CS_AS", "Latin1-General, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 153 on Code Page 1257 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_EBCDIC037_CP1_CS_AS", "Latin1-General, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 210 on Code Page 1252 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_EBCDIC280_CP1_CS_AS", "Latin1-General, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 214 on Code Page 1252 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_EBCDIC285_CP1_CS_AS", "Latin1-General, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 216 on Code Page 1252 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP437_CS_AS", "Latin1-General, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 31 on Code Page 437 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP850_CS_AS", "Latin1-General, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 41 on Code Page 850 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP1_CS_AS", "Latin1-General, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 51 on Code Page 1252 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_AltDiction_CP850_CS_AS", "Latin1-General, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 55 on Code Page 850 for non-Unicode Data"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP1250_CS_AS", "Latin1-General, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 81 on Code Page 1250 for non-Unicode Data"));
                _collationList.Add(new Collation("Latin1_General_CS_AS_WS", "Latin1-General, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Latin1_General_CS_AS_KS", "Latin1-General, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Latin1_General_CS_AS_KS_WS", "Latin1-General, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Latin1_General_100_BIN2", "Latin1-General-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Latin1_General_100_BIN", "Latin1-General-100, binary sort"));
                _collationList.Add(new Collation("Latin1_General_100_CI_AI", "Latin1-General-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Latin1_General_100_CI_AI_WS", "Latin1-General-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Latin1_General_100_CI_AI_KS", "Latin1-General-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Latin1_General_100_CI_AI_KS_WS", "Latin1-General-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Latin1_General_100_CI_AS", "Latin1-General-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Latin1_General_100_CI_AS_WS", "Latin1-General-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Latin1_General_100_CI_AS_KS", "Latin1-General-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Latin1_General_100_CI_AS_KS_WS", "Latin1-General-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Latin1_General_100_CS_AI", "Latin1-General-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Latin1_General_100_CS_AI_WS", "Latin1-General-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Latin1_General_100_CS_AI_KS", "Latin1-General-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Latin1_General_100_CS_AI_KS_WS", "Latin1-General-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Latin1_General_100_CS_AS", "Latin1-General-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Latin1_General_100_CS_AS_WS", "Latin1-General-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Latin1_General_100_CS_AS_KS", "Latin1-General-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Latin1_General_100_CS_AS_KS_WS", "Latin1-General-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Latvian_BIN2", "Latvian, binary code point comparison sort"));
                _collationList.Add(new Collation("Latvian_BIN", "Latvian, binary sort"));
                _collationList.Add(new Collation("Latvian_CI_AI", "Latvian, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Latvian_CI_AI_WS", "Latvian, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Latvian_CI_AI_KS", "Latvian, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Latvian_CI_AI_KS_WS", "Latvian, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Latvian_CI_AS", "Latvian, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Latvian_CP1257_CI_AS", "Latvian, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 158 on Code Page 1257 for non-Unicode Data"));
                _collationList.Add(new Collation("Latvian_CI_AS_WS", "Latvian, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Latvian_CI_AS_KS", "Latvian, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Latvian_CI_AS_KS_WS", "Latvian, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Latvian_CS_AI", "Latvian, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Latvian_CS_AI_WS", "Latvian, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Latvian_CS_AI_KS", "Latvian, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Latvian_CS_AI_KS_WS", "Latvian, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Latvian_CS_AS", "Latvian, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Latvian_CP1257_CS_AS", "Latvian, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 157 on Code Page 1257 for non-Unicode Data"));
                _collationList.Add(new Collation("Latvian_CS_AS_WS", "Latvian, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Latvian_CS_AS_KS", "Latvian, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Latvian_CS_AS_KS_WS", "Latvian, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Latvian_100_BIN2", "Latvian-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Latvian_100_BIN", "Latvian-100, binary sort"));
                _collationList.Add(new Collation("Latvian_100_CI_AI", "Latvian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Latvian_100_CI_AI_WS", "Latvian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Latvian_100_CI_AI_KS", "Latvian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Latvian_100_CI_AI_KS_WS", "Latvian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Latvian_100_CI_AS", "Latvian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Latvian_100_CI_AS_WS", "Latvian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Latvian_100_CI_AS_KS", "Latvian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Latvian_100_CI_AS_KS_WS", "Latvian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Latvian_100_CS_AI", "Latvian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Latvian_100_CS_AI_WS", "Latvian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Latvian_100_CS_AI_KS", "Latvian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Latvian_100_CS_AI_KS_WS", "Latvian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Latvian_100_CS_AS", "Latvian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Latvian_100_CS_AS_WS", "Latvian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Latvian_100_CS_AS_KS", "Latvian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Latvian_100_CS_AS_KS_WS", "Latvian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Lithuanian_BIN2", "Lithuanian, binary code point comparison sort"));
                _collationList.Add(new Collation("Lithuanian_BIN", "Lithuanian, binary sort"));
                _collationList.Add(new Collation("Lithuanian_CI_AI", "Lithuanian, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Lithuanian_CI_AI_WS", "Lithuanian, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Lithuanian_CI_AI_KS", "Lithuanian, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Lithuanian_CI_AI_KS_WS", "Lithuanian, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Lithuanian_CI_AS", "Lithuanian, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Lithuanian_CP1257_CI_AS", "Lithuanian, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 160 on Code Page 1257 for non-Unicode Data"));
                _collationList.Add(new Collation("Lithuanian_CI_AS_WS", "Lithuanian, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Lithuanian_CI_AS_KS", "Lithuanian, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Lithuanian_CI_AS_KS_WS", "Lithuanian, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Lithuanian_CS_AI", "Lithuanian, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Lithuanian_CS_AI_WS", "Lithuanian, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Lithuanian_CS_AI_KS", "Lithuanian, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Lithuanian_CS_AI_KS_WS", "Lithuanian, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Lithuanian_CS_AS", "Lithuanian, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Lithuanian_CP1257_CS_AS", "Lithuanian, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 159 on Code Page 1257 for non-Unicode Data"));
                _collationList.Add(new Collation("Lithuanian_CS_AS_WS", "Lithuanian, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Lithuanian_CS_AS_KS", "Lithuanian, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Lithuanian_CS_AS_KS_WS", "Lithuanian, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Lithuanian_100_BIN2", "Lithuanian-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Lithuanian_100_BIN", "Lithuanian-100, binary sort"));
                _collationList.Add(new Collation("Lithuanian_100_CI_AI", "Lithuanian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Lithuanian_100_CI_AI_WS", "Lithuanian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Lithuanian_100_CI_AI_KS", "Lithuanian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Lithuanian_100_CI_AI_KS_WS", "Lithuanian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Lithuanian_100_CI_AS", "Lithuanian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Lithuanian_100_CI_AS_WS", "Lithuanian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Lithuanian_100_CI_AS_KS", "Lithuanian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Lithuanian_100_CI_AS_KS_WS", "Lithuanian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Lithuanian_100_CS_AI", "Lithuanian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Lithuanian_100_CS_AI_WS", "Lithuanian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Lithuanian_100_CS_AI_KS", "Lithuanian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Lithuanian_100_CS_AI_KS_WS", "Lithuanian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Lithuanian_100_CS_AS", "Lithuanian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Lithuanian_100_CS_AS_WS", "Lithuanian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Lithuanian_100_CS_AS_KS", "Lithuanian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Lithuanian_100_CS_AS_KS_WS", "Lithuanian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_100_BIN2", "Macedonian-FYROM-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Macedonian_FYROM_100_BIN", "Macedonian-FYROM-100, binary sort"));
                _collationList.Add(new Collation("Macedonian_FYROM_100_CI_AI", "Macedonian-FYROM-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_100_CI_AI_WS", "Macedonian-FYROM-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_100_CI_AI_KS", "Macedonian-FYROM-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_100_CI_AI_KS_WS", "Macedonian-FYROM-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_100_CI_AS", "Macedonian-FYROM-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_100_CI_AS_WS", "Macedonian-FYROM-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_100_CI_AS_KS", "Macedonian-FYROM-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_100_CI_AS_KS_WS", "Macedonian-FYROM-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_100_CS_AI", "Macedonian-FYROM-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_100_CS_AI_WS", "Macedonian-FYROM-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_100_CS_AI_KS", "Macedonian-FYROM-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_100_CS_AI_KS_WS", "Macedonian-FYROM-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_100_CS_AS", "Macedonian-FYROM-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_100_CS_AS_WS", "Macedonian-FYROM-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_100_CS_AS_KS", "Macedonian-FYROM-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_100_CS_AS_KS_WS", "Macedonian-FYROM-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_90_BIN2", "Macedonian-FYROM-90, binary code point comparison sort"));
                _collationList.Add(new Collation("Macedonian_FYROM_90_BIN", "Macedonian-FYROM-90, binary sort"));
                _collationList.Add(new Collation("Macedonian_FYROM_90_CI_AI", "Macedonian-FYROM-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_90_CI_AI_WS", "Macedonian-FYROM-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_90_CI_AI_KS", "Macedonian-FYROM-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_90_CI_AI_KS_WS", "Macedonian-FYROM-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_90_CI_AS", "Macedonian-FYROM-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_90_CI_AS_WS", "Macedonian-FYROM-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_90_CI_AS_KS", "Macedonian-FYROM-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_90_CI_AS_KS_WS", "Macedonian-FYROM-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_90_CS_AI", "Macedonian-FYROM-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_90_CS_AI_WS", "Macedonian-FYROM-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_90_CS_AI_KS", "Macedonian-FYROM-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_90_CS_AI_KS_WS", "Macedonian-FYROM-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_90_CS_AS", "Macedonian-FYROM-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_90_CS_AS_WS", "Macedonian-FYROM-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_90_CS_AS_KS", "Macedonian-FYROM-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Macedonian_FYROM_90_CS_AS_KS_WS", "Macedonian-FYROM-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Maltese_100_BIN2", "Maltese-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Maltese_100_BIN", "Maltese-100, binary sort"));
                _collationList.Add(new Collation("Maltese_100_CI_AI", "Maltese-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Maltese_100_CI_AI_WS", "Maltese-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Maltese_100_CI_AI_KS", "Maltese-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Maltese_100_CI_AI_KS_WS", "Maltese-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Maltese_100_CI_AS", "Maltese-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Maltese_100_CI_AS_WS", "Maltese-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Maltese_100_CI_AS_KS", "Maltese-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Maltese_100_CI_AS_KS_WS", "Maltese-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Maltese_100_CS_AI", "Maltese-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Maltese_100_CS_AI_WS", "Maltese-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Maltese_100_CS_AI_KS", "Maltese-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Maltese_100_CS_AI_KS_WS", "Maltese-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Maltese_100_CS_AS", "Maltese-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Maltese_100_CS_AS_WS", "Maltese-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Maltese_100_CS_AS_KS", "Maltese-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Maltese_100_CS_AS_KS_WS", "Maltese-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Maori_100_BIN2", "Maori-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Maori_100_BIN", "Maori-100, binary sort"));
                _collationList.Add(new Collation("Maori_100_CI_AI", "Maori-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Maori_100_CI_AI_WS", "Maori-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Maori_100_CI_AI_KS", "Maori-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Maori_100_CI_AI_KS_WS", "Maori-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Maori_100_CI_AS", "Maori-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Maori_100_CI_AS_WS", "Maori-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Maori_100_CI_AS_KS", "Maori-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Maori_100_CI_AS_KS_WS", "Maori-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Maori_100_CS_AI", "Maori-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Maori_100_CS_AI_WS", "Maori-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Maori_100_CS_AI_KS", "Maori-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Maori_100_CS_AI_KS_WS", "Maori-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Maori_100_CS_AS", "Maori-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Maori_100_CS_AS_WS", "Maori-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Maori_100_CS_AS_KS", "Maori-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Maori_100_CS_AS_KS_WS", "Maori-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Mapudungan_100_BIN2", "Mapudungan-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Mapudungan_100_BIN", "Mapudungan-100, binary sort"));
                _collationList.Add(new Collation("Mapudungan_100_CI_AI", "Mapudungan-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Mapudungan_100_CI_AI_WS", "Mapudungan-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Mapudungan_100_CI_AI_KS", "Mapudungan-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Mapudungan_100_CI_AI_KS_WS", "Mapudungan-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Mapudungan_100_CI_AS", "Mapudungan-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Mapudungan_100_CI_AS_WS", "Mapudungan-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Mapudungan_100_CI_AS_KS", "Mapudungan-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Mapudungan_100_CI_AS_KS_WS", "Mapudungan-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Mapudungan_100_CS_AI", "Mapudungan-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Mapudungan_100_CS_AI_WS", "Mapudungan-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Mapudungan_100_CS_AI_KS", "Mapudungan-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Mapudungan_100_CS_AI_KS_WS", "Mapudungan-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Mapudungan_100_CS_AS", "Mapudungan-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Mapudungan_100_CS_AS_WS", "Mapudungan-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Mapudungan_100_CS_AS_KS", "Mapudungan-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Mapudungan_100_CS_AS_KS_WS", "Mapudungan-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Modern_Spanish_BIN2", "Modern-Spanish, binary code point comparison sort"));
                _collationList.Add(new Collation("Modern_Spanish_BIN", "Modern-Spanish, binary sort"));
                _collationList.Add(new Collation("Modern_Spanish_CI_AI", "Modern-Spanish, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Modern_Spanish_CI_AI_WS", "Modern-Spanish, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Modern_Spanish_CI_AI_KS", "Modern-Spanish, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Modern_Spanish_CI_AI_KS_WS", "Modern-Spanish, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Modern_Spanish_CI_AS", "Modern-Spanish, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Modern_Spanish_CI_AS_WS", "Modern-Spanish, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Modern_Spanish_CI_AS_KS", "Modern-Spanish, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Modern_Spanish_CI_AS_KS_WS", "Modern-Spanish, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Modern_Spanish_CS_AI", "Modern-Spanish, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Modern_Spanish_CS_AI_WS", "Modern-Spanish, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Modern_Spanish_CS_AI_KS", "Modern-Spanish, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Modern_Spanish_CS_AI_KS_WS", "Modern-Spanish, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Modern_Spanish_CS_AS", "Modern-Spanish, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_EBCDIC284_CP1_CS_AS", "Modern-Spanish, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 215 on Code Page 1252 for non-Unicode Data"));
                _collationList.Add(new Collation("Modern_Spanish_CS_AS_WS", "Modern-Spanish, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Modern_Spanish_CS_AS_KS", "Modern-Spanish, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Modern_Spanish_CS_AS_KS_WS", "Modern-Spanish, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Modern_Spanish_100_BIN2", "Modern-Spanish-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Modern_Spanish_100_BIN", "Modern-Spanish-100, binary sort"));
                _collationList.Add(new Collation("Modern_Spanish_100_CI_AI", "Modern-Spanish-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Modern_Spanish_100_CI_AI_WS", "Modern-Spanish-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Modern_Spanish_100_CI_AI_KS", "Modern-Spanish-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Modern_Spanish_100_CI_AI_KS_WS", "Modern-Spanish-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Modern_Spanish_100_CI_AS", "Modern-Spanish-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Modern_Spanish_100_CI_AS_WS", "Modern-Spanish-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Modern_Spanish_100_CI_AS_KS", "Modern-Spanish-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Modern_Spanish_100_CI_AS_KS_WS", "Modern-Spanish-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Modern_Spanish_100_CS_AI", "Modern-Spanish-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Modern_Spanish_100_CS_AI_WS", "Modern-Spanish-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Modern_Spanish_100_CS_AI_KS", "Modern-Spanish-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Modern_Spanish_100_CS_AI_KS_WS", "Modern-Spanish-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Modern_Spanish_100_CS_AS", "Modern-Spanish-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Modern_Spanish_100_CS_AS_WS", "Modern-Spanish-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Modern_Spanish_100_CS_AS_KS", "Modern-Spanish-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Modern_Spanish_100_CS_AS_KS_WS", "Modern-Spanish-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Mohawk_100_BIN2", "Mohawk-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Mohawk_100_BIN", "Mohawk-100, binary sort"));
                _collationList.Add(new Collation("Mohawk_100_CI_AI", "Mohawk-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Mohawk_100_CI_AI_WS", "Mohawk-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Mohawk_100_CI_AI_KS", "Mohawk-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Mohawk_100_CI_AI_KS_WS", "Mohawk-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Mohawk_100_CI_AS", "Mohawk-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Mohawk_100_CI_AS_WS", "Mohawk-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Mohawk_100_CI_AS_KS", "Mohawk-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Mohawk_100_CI_AS_KS_WS", "Mohawk-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Mohawk_100_CS_AI", "Mohawk-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Mohawk_100_CS_AI_WS", "Mohawk-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Mohawk_100_CS_AI_KS", "Mohawk-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Mohawk_100_CS_AI_KS_WS", "Mohawk-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Mohawk_100_CS_AS", "Mohawk-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Mohawk_100_CS_AS_WS", "Mohawk-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Mohawk_100_CS_AS_KS", "Mohawk-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Mohawk_100_CS_AS_KS_WS", "Mohawk-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Nepali_100_BIN2", "Nepali-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Nepali_100_BIN", "Nepali-100, binary sort"));
                _collationList.Add(new Collation("Nepali_100_CI_AI", "Nepali-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Nepali_100_CI_AI_WS", "Nepali-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Nepali_100_CI_AI_KS", "Nepali-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Nepali_100_CI_AI_KS_WS", "Nepali-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Nepali_100_CI_AS", "Nepali-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Nepali_100_CI_AS_WS", "Nepali-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Nepali_100_CI_AS_KS", "Nepali-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Nepali_100_CI_AS_KS_WS", "Nepali-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Nepali_100_CS_AI", "Nepali-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Nepali_100_CS_AI_WS", "Nepali-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Nepali_100_CS_AI_KS", "Nepali-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Nepali_100_CS_AI_KS_WS", "Nepali-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Nepali_100_CS_AS", "Nepali-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Nepali_100_CS_AS_WS", "Nepali-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Nepali_100_CS_AS_KS", "Nepali-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Nepali_100_CS_AS_KS_WS", "Nepali-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Norwegian_100_BIN2", "Norwegian-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Norwegian_100_BIN", "Norwegian-100, binary sort"));
                _collationList.Add(new Collation("Norwegian_100_CI_AI", "Norwegian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Norwegian_100_CI_AI_WS", "Norwegian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Norwegian_100_CI_AI_KS", "Norwegian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Norwegian_100_CI_AI_KS_WS", "Norwegian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Norwegian_100_CI_AS", "Norwegian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Norwegian_100_CI_AS_WS", "Norwegian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Norwegian_100_CI_AS_KS", "Norwegian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Norwegian_100_CI_AS_KS_WS", "Norwegian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Norwegian_100_CS_AI", "Norwegian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Norwegian_100_CS_AI_WS", "Norwegian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Norwegian_100_CS_AI_KS", "Norwegian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Norwegian_100_CS_AI_KS_WS", "Norwegian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Norwegian_100_CS_AS", "Norwegian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Norwegian_100_CS_AS_WS", "Norwegian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Norwegian_100_CS_AS_KS", "Norwegian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Norwegian_100_CS_AS_KS_WS", "Norwegian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Pashto_100_BIN2", "Pashto-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Pashto_100_BIN", "Pashto-100, binary sort"));
                _collationList.Add(new Collation("Pashto_100_CI_AI", "Pashto-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Pashto_100_CI_AI_WS", "Pashto-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Pashto_100_CI_AI_KS", "Pashto-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Pashto_100_CI_AI_KS_WS", "Pashto-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Pashto_100_CI_AS", "Pashto-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Pashto_100_CI_AS_WS", "Pashto-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Pashto_100_CI_AS_KS", "Pashto-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Pashto_100_CI_AS_KS_WS", "Pashto-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Pashto_100_CS_AI", "Pashto-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Pashto_100_CS_AI_WS", "Pashto-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Pashto_100_CS_AI_KS", "Pashto-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Pashto_100_CS_AI_KS_WS", "Pashto-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Pashto_100_CS_AS", "Pashto-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Pashto_100_CS_AS_WS", "Pashto-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Pashto_100_CS_AS_KS", "Pashto-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Pashto_100_CS_AS_KS_WS", "Pashto-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Persian_100_BIN2", "Persian-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Persian_100_BIN", "Persian-100, binary sort"));
                _collationList.Add(new Collation("Persian_100_CI_AI", "Persian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Persian_100_CI_AI_WS", "Persian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Persian_100_CI_AI_KS", "Persian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Persian_100_CI_AI_KS_WS", "Persian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Persian_100_CI_AS", "Persian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Persian_100_CI_AS_WS", "Persian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Persian_100_CI_AS_KS", "Persian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Persian_100_CI_AS_KS_WS", "Persian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Persian_100_CS_AI", "Persian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Persian_100_CS_AI_WS", "Persian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Persian_100_CS_AI_KS", "Persian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Persian_100_CS_AI_KS_WS", "Persian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Persian_100_CS_AS", "Persian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Persian_100_CS_AS_WS", "Persian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Persian_100_CS_AS_KS", "Persian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Persian_100_CS_AS_KS_WS", "Persian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Polish_BIN2", "Polish, binary code point comparison sort"));
                _collationList.Add(new Collation("Polish_BIN", "Polish, binary sort"));
                _collationList.Add(new Collation("Polish_CI_AI", "Polish, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Polish_CI_AI_WS", "Polish, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Polish_CI_AI_KS", "Polish, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Polish_CI_AI_KS_WS", "Polish, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Polish_CI_AS", "Polish, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Polish_CP1250_CI_AS", "Polish, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 88 on Code Page 1250 for non-Unicode Data"));
                _collationList.Add(new Collation("Polish_CI_AS_WS", "Polish, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Polish_CI_AS_KS", "Polish, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Polish_CI_AS_KS_WS", "Polish, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Polish_CS_AI", "Polish, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Polish_CS_AI_WS", "Polish, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Polish_CS_AI_KS", "Polish, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Polish_CS_AI_KS_WS", "Polish, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Polish_CS_AS", "Polish, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Polish_CP1250_CS_AS", "Polish, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 87 on Code Page 1250 for non-Unicode Data"));
                _collationList.Add(new Collation("Polish_CS_AS_WS", "Polish, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Polish_CS_AS_KS", "Polish, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Polish_CS_AS_KS_WS", "Polish, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Polish_100_BIN2", "Polish-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Polish_100_BIN", "Polish-100, binary sort"));
                _collationList.Add(new Collation("Polish_100_CI_AI", "Polish-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Polish_100_CI_AI_WS", "Polish-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Polish_100_CI_AI_KS", "Polish-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Polish_100_CI_AI_KS_WS", "Polish-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Polish_100_CI_AS", "Polish-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Polish_100_CI_AS_WS", "Polish-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Polish_100_CI_AS_KS", "Polish-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Polish_100_CI_AS_KS_WS", "Polish-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Polish_100_CS_AI", "Polish-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Polish_100_CS_AI_WS", "Polish-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Polish_100_CS_AI_KS", "Polish-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Polish_100_CS_AI_KS_WS", "Polish-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Polish_100_CS_AS", "Polish-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Polish_100_CS_AS_WS", "Polish-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Polish_100_CS_AS_KS", "Polish-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Polish_100_CS_AS_KS_WS", "Polish-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Romanian_BIN2", "Romanian, binary code point comparison sort"));
                _collationList.Add(new Collation("Romanian_BIN", "Romanian, binary sort"));
                _collationList.Add(new Collation("Romanian_CI_AI", "Romanian, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Romanian_CI_AI_WS", "Romanian, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Romanian_CI_AI_KS", "Romanian, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Romanian_CI_AI_KS_WS", "Romanian, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Romanian_CI_AS", "Romanian, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Romanian_CP1250_CI_AS", "Romanian, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 90 on Code Page 1250 for non-Unicode Data"));
                _collationList.Add(new Collation("Romanian_CI_AS_WS", "Romanian, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Romanian_CI_AS_KS", "Romanian, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Romanian_CI_AS_KS_WS", "Romanian, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Romanian_CS_AI", "Romanian, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Romanian_CS_AI_WS", "Romanian, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Romanian_CS_AI_KS", "Romanian, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Romanian_CS_AI_KS_WS", "Romanian, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Romanian_CS_AS", "Romanian, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Romanian_CP1250_CS_AS", "Romanian, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 89 on Code Page 1250 for non-Unicode Data"));
                _collationList.Add(new Collation("Romanian_CS_AS_WS", "Romanian, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Romanian_CS_AS_KS", "Romanian, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Romanian_CS_AS_KS_WS", "Romanian, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Romanian_100_BIN2", "Romanian-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Romanian_100_BIN", "Romanian-100, binary sort"));
                _collationList.Add(new Collation("Romanian_100_CI_AI", "Romanian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Romanian_100_CI_AI_WS", "Romanian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Romanian_100_CI_AI_KS", "Romanian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Romanian_100_CI_AI_KS_WS", "Romanian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Romanian_100_CI_AS", "Romanian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Romanian_100_CI_AS_WS", "Romanian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Romanian_100_CI_AS_KS", "Romanian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Romanian_100_CI_AS_KS_WS", "Romanian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Romanian_100_CS_AI", "Romanian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Romanian_100_CS_AI_WS", "Romanian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Romanian_100_CS_AI_KS", "Romanian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Romanian_100_CS_AI_KS_WS", "Romanian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Romanian_100_CS_AS", "Romanian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Romanian_100_CS_AS_WS", "Romanian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Romanian_100_CS_AS_KS", "Romanian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Romanian_100_CS_AS_KS_WS", "Romanian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Romansh_100_BIN2", "Romansh-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Romansh_100_BIN", "Romansh-100, binary sort"));
                _collationList.Add(new Collation("Romansh_100_CI_AI", "Romansh-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Romansh_100_CI_AI_WS", "Romansh-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Romansh_100_CI_AI_KS", "Romansh-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Romansh_100_CI_AI_KS_WS", "Romansh-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Romansh_100_CI_AS", "Romansh-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Romansh_100_CI_AS_WS", "Romansh-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Romansh_100_CI_AS_KS", "Romansh-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Romansh_100_CI_AS_KS_WS", "Romansh-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Romansh_100_CS_AI", "Romansh-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Romansh_100_CS_AI_WS", "Romansh-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Romansh_100_CS_AI_KS", "Romansh-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Romansh_100_CS_AI_KS_WS", "Romansh-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Romansh_100_CS_AS", "Romansh-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Romansh_100_CS_AS_WS", "Romansh-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Romansh_100_CS_AS_KS", "Romansh-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Romansh_100_CS_AS_KS_WS", "Romansh-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Sami_Norway_100_BIN2", "Sami-Norway-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Sami_Norway_100_BIN", "Sami-Norway-100, binary sort"));
                _collationList.Add(new Collation("Sami_Norway_100_CI_AI", "Sami-Norway-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Sami_Norway_100_CI_AI_WS", "Sami-Norway-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Sami_Norway_100_CI_AI_KS", "Sami-Norway-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Sami_Norway_100_CI_AI_KS_WS", "Sami-Norway-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Sami_Norway_100_CI_AS", "Sami-Norway-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Sami_Norway_100_CI_AS_WS", "Sami-Norway-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Sami_Norway_100_CI_AS_KS", "Sami-Norway-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Sami_Norway_100_CI_AS_KS_WS", "Sami-Norway-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Sami_Norway_100_CS_AI", "Sami-Norway-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Sami_Norway_100_CS_AI_WS", "Sami-Norway-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Sami_Norway_100_CS_AI_KS", "Sami-Norway-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Sami_Norway_100_CS_AI_KS_WS", "Sami-Norway-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Sami_Norway_100_CS_AS", "Sami-Norway-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Sami_Norway_100_CS_AS_WS", "Sami-Norway-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Sami_Norway_100_CS_AS_KS", "Sami-Norway-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Sami_Norway_100_CS_AS_KS_WS", "Sami-Norway-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Sami_Sweden_Finland_100_BIN2", "Sami-Sweden-Finland-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Sami_Sweden_Finland_100_BIN", "Sami-Sweden-Finland-100, binary sort"));
                _collationList.Add(new Collation("Sami_Sweden_Finland_100_CI_AI", "Sami-Sweden-Finland-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Sami_Sweden_Finland_100_CI_AI_WS", "Sami-Sweden-Finland-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Sami_Sweden_Finland_100_CI_AI_KS", "Sami-Sweden-Finland-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Sami_Sweden_Finland_100_CI_AI_KS_WS", "Sami-Sweden-Finland-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Sami_Sweden_Finland_100_CI_AS", "Sami-Sweden-Finland-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Sami_Sweden_Finland_100_CI_AS_WS", "Sami-Sweden-Finland-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Sami_Sweden_Finland_100_CI_AS_KS", "Sami-Sweden-Finland-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Sami_Sweden_Finland_100_CI_AS_KS_WS", "Sami-Sweden-Finland-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Sami_Sweden_Finland_100_CS_AI", "Sami-Sweden-Finland-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Sami_Sweden_Finland_100_CS_AI_WS", "Sami-Sweden-Finland-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Sami_Sweden_Finland_100_CS_AI_KS", "Sami-Sweden-Finland-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Sami_Sweden_Finland_100_CS_AI_KS_WS", "Sami-Sweden-Finland-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Sami_Sweden_Finland_100_CS_AS", "Sami-Sweden-Finland-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Sami_Sweden_Finland_100_CS_AS_WS", "Sami-Sweden-Finland-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Sami_Sweden_Finland_100_CS_AS_KS", "Sami-Sweden-Finland-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Sami_Sweden_Finland_100_CS_AS_KS_WS", "Sami-Sweden-Finland-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Serbian_Cyrillic_100_BIN2", "Serbian-Cyrillic-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Serbian_Cyrillic_100_BIN", "Serbian-Cyrillic-100, binary sort"));
                _collationList.Add(new Collation("Serbian_Cyrillic_100_CI_AI", "Serbian-Cyrillic-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Serbian_Cyrillic_100_CI_AI_WS", "Serbian-Cyrillic-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Serbian_Cyrillic_100_CI_AI_KS", "Serbian-Cyrillic-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Serbian_Cyrillic_100_CI_AI_KS_WS", "Serbian-Cyrillic-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Serbian_Cyrillic_100_CI_AS", "Serbian-Cyrillic-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Serbian_Cyrillic_100_CI_AS_WS", "Serbian-Cyrillic-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Serbian_Cyrillic_100_CI_AS_KS", "Serbian-Cyrillic-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Serbian_Cyrillic_100_CI_AS_KS_WS", "Serbian-Cyrillic-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Serbian_Cyrillic_100_CS_AI", "Serbian-Cyrillic-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Serbian_Cyrillic_100_CS_AI_WS", "Serbian-Cyrillic-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Serbian_Cyrillic_100_CS_AI_KS", "Serbian-Cyrillic-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Serbian_Cyrillic_100_CS_AI_KS_WS", "Serbian-Cyrillic-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Serbian_Cyrillic_100_CS_AS", "Serbian-Cyrillic-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Serbian_Cyrillic_100_CS_AS_WS", "Serbian-Cyrillic-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Serbian_Cyrillic_100_CS_AS_KS", "Serbian-Cyrillic-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Serbian_Cyrillic_100_CS_AS_KS_WS", "Serbian-Cyrillic-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Serbian_Latin_100_BIN2", "Serbian-Latin-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Serbian_Latin_100_BIN", "Serbian-Latin-100, binary sort"));
                _collationList.Add(new Collation("Serbian_Latin_100_CI_AI", "Serbian-Latin-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Serbian_Latin_100_CI_AI_WS", "Serbian-Latin-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Serbian_Latin_100_CI_AI_KS", "Serbian-Latin-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Serbian_Latin_100_CI_AI_KS_WS", "Serbian-Latin-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Serbian_Latin_100_CI_AS", "Serbian-Latin-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Serbian_Latin_100_CI_AS_WS", "Serbian-Latin-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Serbian_Latin_100_CI_AS_KS", "Serbian-Latin-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Serbian_Latin_100_CI_AS_KS_WS", "Serbian-Latin-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Serbian_Latin_100_CS_AI", "Serbian-Latin-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Serbian_Latin_100_CS_AI_WS", "Serbian-Latin-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Serbian_Latin_100_CS_AI_KS", "Serbian-Latin-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Serbian_Latin_100_CS_AI_KS_WS", "Serbian-Latin-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Serbian_Latin_100_CS_AS", "Serbian-Latin-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Serbian_Latin_100_CS_AS_WS", "Serbian-Latin-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Serbian_Latin_100_CS_AS_KS", "Serbian-Latin-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Serbian_Latin_100_CS_AS_KS_WS", "Serbian-Latin-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovak_BIN2", "Slovak, binary code point comparison sort"));
                _collationList.Add(new Collation("Slovak_BIN", "Slovak, binary sort"));
                _collationList.Add(new Collation("Slovak_CI_AI", "Slovak, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovak_CI_AI_WS", "Slovak, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovak_CI_AI_KS", "Slovak, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovak_CI_AI_KS_WS", "Slovak, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovak_CI_AS", "Slovak, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Slovak_CP1250_CI_AS", "Slovak, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 94 on Code Page 1250 for non-Unicode Data"));
                _collationList.Add(new Collation("Slovak_CI_AS_WS", "Slovak, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovak_CI_AS_KS", "Slovak, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovak_CI_AS_KS_WS", "Slovak, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovak_CS_AI", "Slovak, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovak_CS_AI_WS", "Slovak, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovak_CS_AI_KS", "Slovak, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovak_CS_AI_KS_WS", "Slovak, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovak_CS_AS", "Slovak, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Slovak_CP1250_CS_AS", "Slovak, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 93 on Code Page 1250 for non-Unicode Data"));
                _collationList.Add(new Collation("Slovak_CS_AS_WS", "Slovak, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovak_CS_AS_KS", "Slovak, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovak_CS_AS_KS_WS", "Slovak, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovak_100_BIN2", "Slovak-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Slovak_100_BIN", "Slovak-100, binary sort"));
                _collationList.Add(new Collation("Slovak_100_CI_AI", "Slovak-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovak_100_CI_AI_WS", "Slovak-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovak_100_CI_AI_KS", "Slovak-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovak_100_CI_AI_KS_WS", "Slovak-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovak_100_CI_AS", "Slovak-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovak_100_CI_AS_WS", "Slovak-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovak_100_CI_AS_KS", "Slovak-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovak_100_CI_AS_KS_WS", "Slovak-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovak_100_CS_AI", "Slovak-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovak_100_CS_AI_WS", "Slovak-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovak_100_CS_AI_KS", "Slovak-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovak_100_CS_AI_KS_WS", "Slovak-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovak_100_CS_AS", "Slovak-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovak_100_CS_AS_WS", "Slovak-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovak_100_CS_AS_KS", "Slovak-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovak_100_CS_AS_KS_WS", "Slovak-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovenian_BIN2", "Slovenian, binary code point comparison sort"));
                _collationList.Add(new Collation("Slovenian_BIN", "Slovenian, binary sort"));
                _collationList.Add(new Collation("Slovenian_CI_AI", "Slovenian, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovenian_CI_AI_WS", "Slovenian, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovenian_CI_AI_KS", "Slovenian, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovenian_CI_AI_KS_WS", "Slovenian, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovenian_CI_AS", "Slovenian, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Slovenian_CP1250_CI_AS", "Slovenian, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 96 on Code Page 1250 for non-Unicode Data"));
                _collationList.Add(new Collation("Slovenian_CI_AS_WS", "Slovenian, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovenian_CI_AS_KS", "Slovenian, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovenian_CI_AS_KS_WS", "Slovenian, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovenian_CS_AI", "Slovenian, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovenian_CS_AI_WS", "Slovenian, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovenian_CS_AI_KS", "Slovenian, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovenian_CS_AI_KS_WS", "Slovenian, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovenian_CS_AS", "Slovenian, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Slovenian_CP1250_CS_AS", "Slovenian, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 95 on Code Page 1250 for non-Unicode Data"));
                _collationList.Add(new Collation("Slovenian_CS_AS_WS", "Slovenian, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovenian_CS_AS_KS", "Slovenian, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovenian_CS_AS_KS_WS", "Slovenian, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovenian_100_BIN2", "Slovenian-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Slovenian_100_BIN", "Slovenian-100, binary sort"));
                _collationList.Add(new Collation("Slovenian_100_CI_AI", "Slovenian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovenian_100_CI_AI_WS", "Slovenian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovenian_100_CI_AI_KS", "Slovenian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovenian_100_CI_AI_KS_WS", "Slovenian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovenian_100_CI_AS", "Slovenian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovenian_100_CI_AS_WS", "Slovenian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovenian_100_CI_AS_KS", "Slovenian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovenian_100_CI_AS_KS_WS", "Slovenian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovenian_100_CS_AI", "Slovenian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovenian_100_CS_AI_WS", "Slovenian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovenian_100_CS_AI_KS", "Slovenian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovenian_100_CS_AI_KS_WS", "Slovenian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovenian_100_CS_AS", "Slovenian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovenian_100_CS_AS_WS", "Slovenian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Slovenian_100_CS_AS_KS", "Slovenian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Slovenian_100_CS_AS_KS_WS", "Slovenian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Syriac_100_BIN2", "Syriac-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Syriac_100_BIN", "Syriac-100, binary sort"));
                _collationList.Add(new Collation("Syriac_100_CI_AI", "Syriac-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Syriac_100_CI_AI_WS", "Syriac-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Syriac_100_CI_AI_KS", "Syriac-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Syriac_100_CI_AI_KS_WS", "Syriac-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Syriac_100_CI_AS", "Syriac-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Syriac_100_CI_AS_WS", "Syriac-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Syriac_100_CI_AS_KS", "Syriac-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Syriac_100_CI_AS_KS_WS", "Syriac-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Syriac_100_CS_AI", "Syriac-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Syriac_100_CS_AI_WS", "Syriac-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Syriac_100_CS_AI_KS", "Syriac-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Syriac_100_CS_AI_KS_WS", "Syriac-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Syriac_100_CS_AS", "Syriac-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Syriac_100_CS_AS_WS", "Syriac-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Syriac_100_CS_AS_KS", "Syriac-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Syriac_100_CS_AS_KS_WS", "Syriac-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Syriac_90_BIN2", "Syriac-90, binary code point comparison sort"));
                _collationList.Add(new Collation("Syriac_90_BIN", "Syriac-90, binary sort"));
                _collationList.Add(new Collation("Syriac_90_CI_AI", "Syriac-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Syriac_90_CI_AI_WS", "Syriac-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Syriac_90_CI_AI_KS", "Syriac-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Syriac_90_CI_AI_KS_WS", "Syriac-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Syriac_90_CI_AS", "Syriac-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Syriac_90_CI_AS_WS", "Syriac-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Syriac_90_CI_AS_KS", "Syriac-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Syriac_90_CI_AS_KS_WS", "Syriac-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Syriac_90_CS_AI", "Syriac-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Syriac_90_CS_AI_WS", "Syriac-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Syriac_90_CS_AI_KS", "Syriac-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Syriac_90_CS_AI_KS_WS", "Syriac-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Syriac_90_CS_AS", "Syriac-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Syriac_90_CS_AS_WS", "Syriac-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Syriac_90_CS_AS_KS", "Syriac-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Syriac_90_CS_AS_KS_WS", "Syriac-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Tamazight_100_BIN2", "Tamazight-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Tamazight_100_BIN", "Tamazight-100, binary sort"));
                _collationList.Add(new Collation("Tamazight_100_CI_AI", "Tamazight-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Tamazight_100_CI_AI_WS", "Tamazight-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Tamazight_100_CI_AI_KS", "Tamazight-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Tamazight_100_CI_AI_KS_WS", "Tamazight-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Tamazight_100_CI_AS", "Tamazight-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Tamazight_100_CI_AS_WS", "Tamazight-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Tamazight_100_CI_AS_KS", "Tamazight-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Tamazight_100_CI_AS_KS_WS", "Tamazight-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Tamazight_100_CS_AI", "Tamazight-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Tamazight_100_CS_AI_WS", "Tamazight-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Tamazight_100_CS_AI_KS", "Tamazight-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Tamazight_100_CS_AI_KS_WS", "Tamazight-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Tamazight_100_CS_AS", "Tamazight-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Tamazight_100_CS_AS_WS", "Tamazight-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Tamazight_100_CS_AS_KS", "Tamazight-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Tamazight_100_CS_AS_KS_WS", "Tamazight-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Tatar_100_BIN2", "Tatar-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Tatar_100_BIN", "Tatar-100, binary sort"));
                _collationList.Add(new Collation("Tatar_100_CI_AI", "Tatar-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Tatar_100_CI_AI_WS", "Tatar-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Tatar_100_CI_AI_KS", "Tatar-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Tatar_100_CI_AI_KS_WS", "Tatar-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Tatar_100_CI_AS", "Tatar-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Tatar_100_CI_AS_WS", "Tatar-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Tatar_100_CI_AS_KS", "Tatar-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Tatar_100_CI_AS_KS_WS", "Tatar-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Tatar_100_CS_AI", "Tatar-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Tatar_100_CS_AI_WS", "Tatar-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Tatar_100_CS_AI_KS", "Tatar-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Tatar_100_CS_AI_KS_WS", "Tatar-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Tatar_100_CS_AS", "Tatar-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Tatar_100_CS_AS_WS", "Tatar-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Tatar_100_CS_AS_KS", "Tatar-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Tatar_100_CS_AS_KS_WS", "Tatar-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Tatar_90_BIN2", "Tatar-90, binary code point comparison sort"));
                _collationList.Add(new Collation("Tatar_90_BIN", "Tatar-90, binary sort"));
                _collationList.Add(new Collation("Tatar_90_CI_AI", "Tatar-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Tatar_90_CI_AI_WS", "Tatar-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Tatar_90_CI_AI_KS", "Tatar-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Tatar_90_CI_AI_KS_WS", "Tatar-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Tatar_90_CI_AS", "Tatar-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Tatar_90_CI_AS_WS", "Tatar-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Tatar_90_CI_AS_KS", "Tatar-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Tatar_90_CI_AS_KS_WS", "Tatar-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Tatar_90_CS_AI", "Tatar-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Tatar_90_CS_AI_WS", "Tatar-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Tatar_90_CS_AI_KS", "Tatar-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Tatar_90_CS_AI_KS_WS", "Tatar-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Tatar_90_CS_AS", "Tatar-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Tatar_90_CS_AS_WS", "Tatar-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Tatar_90_CS_AS_KS", "Tatar-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Tatar_90_CS_AS_KS_WS", "Tatar-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Thai_BIN2", "Thai, binary code point comparison sort"));
                _collationList.Add(new Collation("Thai_BIN", "Thai, binary sort"));
                _collationList.Add(new Collation("Thai_CI_AI", "Thai, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Thai_CI_AI_WS", "Thai, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Thai_CI_AI_KS", "Thai, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Thai_CI_AI_KS_WS", "Thai, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Thai_CI_AS", "Thai, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Thai_CI_AS_WS", "Thai, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Thai_CI_AS_KS", "Thai, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Thai_CI_AS_KS_WS", "Thai, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Thai_CS_AI", "Thai, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Thai_CS_AI_WS", "Thai, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Thai_CS_AI_KS", "Thai, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Thai_CS_AI_KS_WS", "Thai, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Thai_CS_AS", "Thai, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Thai_CS_AS_WS", "Thai, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Thai_CS_AS_KS", "Thai, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Thai_CS_AS_KS_WS", "Thai, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Thai_100_BIN2", "Thai-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Thai_100_BIN", "Thai-100, binary sort"));
                _collationList.Add(new Collation("Thai_100_CI_AI", "Thai-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Thai_100_CI_AI_WS", "Thai-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Thai_100_CI_AI_KS", "Thai-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Thai_100_CI_AI_KS_WS", "Thai-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Thai_100_CI_AS", "Thai-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Thai_100_CI_AS_WS", "Thai-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Thai_100_CI_AS_KS", "Thai-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Thai_100_CI_AS_KS_WS", "Thai-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Thai_100_CS_AI", "Thai-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Thai_100_CS_AI_WS", "Thai-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Thai_100_CS_AI_KS", "Thai-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Thai_100_CS_AI_KS_WS", "Thai-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Thai_100_CS_AS", "Thai-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Thai_100_CS_AS_WS", "Thai-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Thai_100_CS_AS_KS", "Thai-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Thai_100_CS_AS_KS_WS", "Thai-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Tibetan_100_BIN2", "Tibetan-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Tibetan_100_BIN", "Tibetan-100, binary sort"));
                _collationList.Add(new Collation("Tibetan_100_CI_AI", "Tibetan-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Tibetan_100_CI_AI_WS", "Tibetan-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Tibetan_100_CI_AI_KS", "Tibetan-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Tibetan_100_CI_AI_KS_WS", "Tibetan-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Tibetan_100_CI_AS", "Tibetan-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Tibetan_100_CI_AS_WS", "Tibetan-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Tibetan_100_CI_AS_KS", "Tibetan-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Tibetan_100_CI_AS_KS_WS", "Tibetan-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Tibetan_100_CS_AI", "Tibetan-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Tibetan_100_CS_AI_WS", "Tibetan-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Tibetan_100_CS_AI_KS", "Tibetan-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Tibetan_100_CS_AI_KS_WS", "Tibetan-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Tibetan_100_CS_AS", "Tibetan-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Tibetan_100_CS_AS_WS", "Tibetan-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Tibetan_100_CS_AS_KS", "Tibetan-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Tibetan_100_CS_AS_KS_WS", "Tibetan-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_BIN2", "Traditional-Spanish, binary code point comparison sort"));
                _collationList.Add(new Collation("Traditional_Spanish_BIN", "Traditional-Spanish, binary sort"));
                _collationList.Add(new Collation("Traditional_Spanish_CI_AI", "Traditional-Spanish, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_CI_AI_WS", "Traditional-Spanish, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_CI_AI_KS", "Traditional-Spanish, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_CI_AI_KS_WS", "Traditional-Spanish, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_CI_AS", "Traditional-Spanish, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_CI_AS_WS", "Traditional-Spanish, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_CI_AS_KS", "Traditional-Spanish, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_CI_AS_KS_WS", "Traditional-Spanish, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_CS_AI", "Traditional-Spanish, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_CS_AI_WS", "Traditional-Spanish, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_CS_AI_KS", "Traditional-Spanish, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_CS_AI_KS_WS", "Traditional-Spanish, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_CS_AS", "Traditional-Spanish, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_CS_AS_WS", "Traditional-Spanish, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_CS_AS_KS", "Traditional-Spanish, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_CS_AS_KS_WS", "Traditional-Spanish, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_100_BIN2", "Traditional-Spanish-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Traditional_Spanish_100_BIN", "Traditional-Spanish-100, binary sort"));
                _collationList.Add(new Collation("Traditional_Spanish_100_CI_AI", "Traditional-Spanish-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_100_CI_AI_WS", "Traditional-Spanish-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_100_CI_AI_KS", "Traditional-Spanish-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_100_CI_AI_KS_WS", "Traditional-Spanish-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_100_CI_AS", "Traditional-Spanish-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_100_CI_AS_WS", "Traditional-Spanish-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_100_CI_AS_KS", "Traditional-Spanish-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_100_CI_AS_KS_WS", "Traditional-Spanish-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_100_CS_AI", "Traditional-Spanish-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_100_CS_AI_WS", "Traditional-Spanish-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_100_CS_AI_KS", "Traditional-Spanish-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_100_CS_AI_KS_WS", "Traditional-Spanish-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_100_CS_AS", "Traditional-Spanish-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_100_CS_AS_WS", "Traditional-Spanish-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_100_CS_AS_KS", "Traditional-Spanish-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Traditional_Spanish_100_CS_AS_KS_WS", "Traditional-Spanish-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkish_BIN2", "Turkish, binary code point comparison sort"));
                _collationList.Add(new Collation("Turkish_BIN", "Turkish, binary sort"));
                _collationList.Add(new Collation("Turkish_CI_AI", "Turkish, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkish_CI_AI_WS", "Turkish, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkish_CI_AI_KS", "Turkish, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkish_CI_AI_KS_WS", "Turkish, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkish_CI_AS", "Turkish, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP1254_CI_AS", "Turkish, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 130 on Code Page 1254 for non-Unicode Data"));
                _collationList.Add(new Collation("Turkish_CI_AS_WS", "Turkish, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkish_CI_AS_KS", "Turkish, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkish_CI_AS_KS_WS", "Turkish, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkish_CS_AI", "Turkish, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkish_CS_AI_WS", "Turkish, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkish_CS_AI_KS", "Turkish, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkish_CS_AI_KS_WS", "Turkish, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkish_CS_AS", "Turkish, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Latin1_General_CP1254_CS_AS", "Turkish, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 129 on Code Page 1254 for non-Unicode Data"));
                _collationList.Add(new Collation("Turkish_CS_AS_WS", "Turkish, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkish_CS_AS_KS", "Turkish, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkish_CS_AS_KS_WS", "Turkish, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkish_100_BIN2", "Turkish-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Turkish_100_BIN", "Turkish-100, binary sort"));
                _collationList.Add(new Collation("Turkish_100_CI_AI", "Turkish-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkish_100_CI_AI_WS", "Turkish-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkish_100_CI_AI_KS", "Turkish-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkish_100_CI_AI_KS_WS", "Turkish-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkish_100_CI_AS", "Turkish-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkish_100_CI_AS_WS", "Turkish-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkish_100_CI_AS_KS", "Turkish-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkish_100_CI_AS_KS_WS", "Turkish-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkish_100_CS_AI", "Turkish-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkish_100_CS_AI_WS", "Turkish-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkish_100_CS_AI_KS", "Turkish-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkish_100_CS_AI_KS_WS", "Turkish-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkish_100_CS_AS", "Turkish-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkish_100_CS_AS_WS", "Turkish-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkish_100_CS_AS_KS", "Turkish-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkish_100_CS_AS_KS_WS", "Turkish-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkmen_100_BIN2", "Turkmen-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Turkmen_100_BIN", "Turkmen-100, binary sort"));
                _collationList.Add(new Collation("Turkmen_100_CI_AI", "Turkmen-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkmen_100_CI_AI_WS", "Turkmen-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkmen_100_CI_AI_KS", "Turkmen-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkmen_100_CI_AI_KS_WS", "Turkmen-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkmen_100_CI_AS", "Turkmen-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkmen_100_CI_AS_WS", "Turkmen-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkmen_100_CI_AS_KS", "Turkmen-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkmen_100_CI_AS_KS_WS", "Turkmen-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkmen_100_CS_AI", "Turkmen-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkmen_100_CS_AI_WS", "Turkmen-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkmen_100_CS_AI_KS", "Turkmen-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkmen_100_CS_AI_KS_WS", "Turkmen-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkmen_100_CS_AS", "Turkmen-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkmen_100_CS_AS_WS", "Turkmen-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Turkmen_100_CS_AS_KS", "Turkmen-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Turkmen_100_CS_AS_KS_WS", "Turkmen-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Uighur_100_BIN2", "Uighur-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Uighur_100_BIN", "Uighur-100, binary sort"));
                _collationList.Add(new Collation("Uighur_100_CI_AI", "Uighur-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Uighur_100_CI_AI_WS", "Uighur-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Uighur_100_CI_AI_KS", "Uighur-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Uighur_100_CI_AI_KS_WS", "Uighur-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Uighur_100_CI_AS", "Uighur-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Uighur_100_CI_AS_WS", "Uighur-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Uighur_100_CI_AS_KS", "Uighur-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Uighur_100_CI_AS_KS_WS", "Uighur-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Uighur_100_CS_AI", "Uighur-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Uighur_100_CS_AI_WS", "Uighur-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Uighur_100_CS_AI_KS", "Uighur-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Uighur_100_CS_AI_KS_WS", "Uighur-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Uighur_100_CS_AS", "Uighur-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Uighur_100_CS_AS_WS", "Uighur-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Uighur_100_CS_AS_KS", "Uighur-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Uighur_100_CS_AS_KS_WS", "Uighur-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Ukrainian_BIN2", "Ukrainian, binary code point comparison sort"));
                _collationList.Add(new Collation("Ukrainian_BIN", "Ukrainian, binary sort"));
                _collationList.Add(new Collation("Ukrainian_CI_AI", "Ukrainian, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Ukrainian_CI_AI_WS", "Ukrainian, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Ukrainian_CI_AI_KS", "Ukrainian, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Ukrainian_CI_AI_KS_WS", "Ukrainian, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Ukrainian_CI_AS", "Ukrainian, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Ukrainian_CP1251_CI_AS", "Ukrainian, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 108 on Code Page 1251 for non-Unicode Data"));
                _collationList.Add(new Collation("Ukrainian_CI_AS_WS", "Ukrainian, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Ukrainian_CI_AS_KS", "Ukrainian, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Ukrainian_CI_AS_KS_WS", "Ukrainian, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Ukrainian_CS_AI", "Ukrainian, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Ukrainian_CS_AI_WS", "Ukrainian, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Ukrainian_CS_AI_KS", "Ukrainian, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Ukrainian_CS_AI_KS_WS", "Ukrainian, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Ukrainian_CS_AS", "Ukrainian, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("SQL_Ukrainian_CP1251_CS_AS", "Ukrainian, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive for Unicode Data, SQL Server Sort Order 107 on Code Page 1251 for non-Unicode Data"));
                _collationList.Add(new Collation("Ukrainian_CS_AS_WS", "Ukrainian, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Ukrainian_CS_AS_KS", "Ukrainian, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Ukrainian_CS_AS_KS_WS", "Ukrainian, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Ukrainian_100_BIN2", "Ukrainian-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Ukrainian_100_BIN", "Ukrainian-100, binary sort"));
                _collationList.Add(new Collation("Ukrainian_100_CI_AI", "Ukrainian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Ukrainian_100_CI_AI_WS", "Ukrainian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Ukrainian_100_CI_AI_KS", "Ukrainian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Ukrainian_100_CI_AI_KS_WS", "Ukrainian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Ukrainian_100_CI_AS", "Ukrainian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Ukrainian_100_CI_AS_WS", "Ukrainian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Ukrainian_100_CI_AS_KS", "Ukrainian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Ukrainian_100_CI_AS_KS_WS", "Ukrainian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Ukrainian_100_CS_AI", "Ukrainian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Ukrainian_100_CS_AI_WS", "Ukrainian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Ukrainian_100_CS_AI_KS", "Ukrainian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Ukrainian_100_CS_AI_KS_WS", "Ukrainian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Ukrainian_100_CS_AS", "Ukrainian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Ukrainian_100_CS_AS_WS", "Ukrainian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Ukrainian_100_CS_AS_KS", "Ukrainian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Ukrainian_100_CS_AS_KS_WS", "Ukrainian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Upper_Sorbian_100_BIN2", "Upper-Sorbian-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Upper_Sorbian_100_BIN", "Upper-Sorbian-100, binary sort"));
                _collationList.Add(new Collation("Upper_Sorbian_100_CI_AI", "Upper-Sorbian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Upper_Sorbian_100_CI_AI_WS", "Upper-Sorbian-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Upper_Sorbian_100_CI_AI_KS", "Upper-Sorbian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Upper_Sorbian_100_CI_AI_KS_WS", "Upper-Sorbian-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Upper_Sorbian_100_CI_AS", "Upper-Sorbian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Upper_Sorbian_100_CI_AS_WS", "Upper-Sorbian-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Upper_Sorbian_100_CI_AS_KS", "Upper-Sorbian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Upper_Sorbian_100_CI_AS_KS_WS", "Upper-Sorbian-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Upper_Sorbian_100_CS_AI", "Upper-Sorbian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Upper_Sorbian_100_CS_AI_WS", "Upper-Sorbian-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Upper_Sorbian_100_CS_AI_KS", "Upper-Sorbian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Upper_Sorbian_100_CS_AI_KS_WS", "Upper-Sorbian-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Upper_Sorbian_100_CS_AS", "Upper-Sorbian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Upper_Sorbian_100_CS_AS_WS", "Upper-Sorbian-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Upper_Sorbian_100_CS_AS_KS", "Upper-Sorbian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Upper_Sorbian_100_CS_AS_KS_WS", "Upper-Sorbian-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Urdu_100_BIN2", "Urdu-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Urdu_100_BIN", "Urdu-100, binary sort"));
                _collationList.Add(new Collation("Urdu_100_CI_AI", "Urdu-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Urdu_100_CI_AI_WS", "Urdu-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Urdu_100_CI_AI_KS", "Urdu-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Urdu_100_CI_AI_KS_WS", "Urdu-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Urdu_100_CI_AS", "Urdu-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Urdu_100_CI_AS_WS", "Urdu-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Urdu_100_CI_AS_KS", "Urdu-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Urdu_100_CI_AS_KS_WS", "Urdu-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Urdu_100_CS_AI", "Urdu-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Urdu_100_CS_AI_WS", "Urdu-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Urdu_100_CS_AI_KS", "Urdu-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Urdu_100_CS_AI_KS_WS", "Urdu-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Urdu_100_CS_AS", "Urdu-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Urdu_100_CS_AS_WS", "Urdu-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Urdu_100_CS_AS_KS", "Urdu-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Urdu_100_CS_AS_KS_WS", "Urdu-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_100_BIN2", "Uzbek-Latin-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Uzbek_Latin_100_BIN", "Uzbek-Latin-100, binary sort"));
                _collationList.Add(new Collation("Uzbek_Latin_100_CI_AI", "Uzbek-Latin-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_100_CI_AI_WS", "Uzbek-Latin-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_100_CI_AI_KS", "Uzbek-Latin-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_100_CI_AI_KS_WS", "Uzbek-Latin-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_100_CI_AS", "Uzbek-Latin-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_100_CI_AS_WS", "Uzbek-Latin-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_100_CI_AS_KS", "Uzbek-Latin-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_100_CI_AS_KS_WS", "Uzbek-Latin-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_100_CS_AI", "Uzbek-Latin-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_100_CS_AI_WS", "Uzbek-Latin-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_100_CS_AI_KS", "Uzbek-Latin-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_100_CS_AI_KS_WS", "Uzbek-Latin-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_100_CS_AS", "Uzbek-Latin-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_100_CS_AS_WS", "Uzbek-Latin-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_100_CS_AS_KS", "Uzbek-Latin-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_100_CS_AS_KS_WS", "Uzbek-Latin-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_90_BIN2", "Uzbek-Latin-90, binary code point comparison sort"));
                _collationList.Add(new Collation("Uzbek_Latin_90_BIN", "Uzbek-Latin-90, binary sort"));
                _collationList.Add(new Collation("Uzbek_Latin_90_CI_AI", "Uzbek-Latin-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_90_CI_AI_WS", "Uzbek-Latin-90, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_90_CI_AI_KS", "Uzbek-Latin-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_90_CI_AI_KS_WS", "Uzbek-Latin-90, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_90_CI_AS", "Uzbek-Latin-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_90_CI_AS_WS", "Uzbek-Latin-90, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_90_CI_AS_KS", "Uzbek-Latin-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_90_CI_AS_KS_WS", "Uzbek-Latin-90, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_90_CS_AI", "Uzbek-Latin-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_90_CS_AI_WS", "Uzbek-Latin-90, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_90_CS_AI_KS", "Uzbek-Latin-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_90_CS_AI_KS_WS", "Uzbek-Latin-90, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_90_CS_AS", "Uzbek-Latin-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_90_CS_AS_WS", "Uzbek-Latin-90, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_90_CS_AS_KS", "Uzbek-Latin-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Uzbek_Latin_90_CS_AS_KS_WS", "Uzbek-Latin-90, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Vietnamese_BIN2", "Vietnamese, binary code point comparison sort"));
                _collationList.Add(new Collation("Vietnamese_BIN", "Vietnamese, binary sort"));
                _collationList.Add(new Collation("Vietnamese_CI_AI", "Vietnamese, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Vietnamese_CI_AI_WS", "Vietnamese, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Vietnamese_CI_AI_KS", "Vietnamese, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Vietnamese_CI_AI_KS_WS", "Vietnamese, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Vietnamese_CI_AS", "Vietnamese, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Vietnamese_CI_AS_WS", "Vietnamese, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Vietnamese_CI_AS_KS", "Vietnamese, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Vietnamese_CI_AS_KS_WS", "Vietnamese, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Vietnamese_CS_AI", "Vietnamese, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Vietnamese_CS_AI_WS", "Vietnamese, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Vietnamese_CS_AI_KS", "Vietnamese, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Vietnamese_CS_AI_KS_WS", "Vietnamese, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Vietnamese_CS_AS", "Vietnamese, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Vietnamese_CS_AS_WS", "Vietnamese, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Vietnamese_CS_AS_KS", "Vietnamese, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Vietnamese_CS_AS_KS_WS", "Vietnamese, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Vietnamese_100_BIN2", "Vietnamese-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Vietnamese_100_BIN", "Vietnamese-100, binary sort"));
                _collationList.Add(new Collation("Vietnamese_100_CI_AI", "Vietnamese-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Vietnamese_100_CI_AI_WS", "Vietnamese-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Vietnamese_100_CI_AI_KS", "Vietnamese-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Vietnamese_100_CI_AI_KS_WS", "Vietnamese-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Vietnamese_100_CI_AS", "Vietnamese-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Vietnamese_100_CI_AS_WS", "Vietnamese-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Vietnamese_100_CI_AS_KS", "Vietnamese-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Vietnamese_100_CI_AS_KS_WS", "Vietnamese-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Vietnamese_100_CS_AI", "Vietnamese-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Vietnamese_100_CS_AI_WS", "Vietnamese-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Vietnamese_100_CS_AI_KS", "Vietnamese-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Vietnamese_100_CS_AI_KS_WS", "Vietnamese-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Vietnamese_100_CS_AS", "Vietnamese-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Vietnamese_100_CS_AS_WS", "Vietnamese-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Vietnamese_100_CS_AS_KS", "Vietnamese-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Vietnamese_100_CS_AS_KS_WS", "Vietnamese-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Welsh_100_BIN2", "Welsh-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Welsh_100_BIN", "Welsh-100, binary sort"));
                _collationList.Add(new Collation("Welsh_100_CI_AI", "Welsh-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Welsh_100_CI_AI_WS", "Welsh-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Welsh_100_CI_AI_KS", "Welsh-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Welsh_100_CI_AI_KS_WS", "Welsh-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Welsh_100_CI_AS", "Welsh-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Welsh_100_CI_AS_WS", "Welsh-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Welsh_100_CI_AS_KS", "Welsh-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Welsh_100_CI_AS_KS_WS", "Welsh-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Welsh_100_CS_AI", "Welsh-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Welsh_100_CS_AI_WS", "Welsh-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Welsh_100_CS_AI_KS", "Welsh-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Welsh_100_CS_AI_KS_WS", "Welsh-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Welsh_100_CS_AS", "Welsh-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Welsh_100_CS_AS_WS", "Welsh-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Welsh_100_CS_AS_KS", "Welsh-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Welsh_100_CS_AS_KS_WS", "Welsh-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Yakut_100_BIN2", "Yakut-100, binary code point comparison sort"));
                _collationList.Add(new Collation("Yakut_100_BIN", "Yakut-100, binary sort"));
                _collationList.Add(new Collation("Yakut_100_CI_AI", "Yakut-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Yakut_100_CI_AI_WS", "Yakut-100, case-insensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Yakut_100_CI_AI_KS", "Yakut-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Yakut_100_CI_AI_KS_WS", "Yakut-100, case-insensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Yakut_100_CI_AS", "Yakut-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Yakut_100_CI_AS_WS", "Yakut-100, case-insensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Yakut_100_CI_AS_KS", "Yakut-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Yakut_100_CI_AS_KS_WS", "Yakut-100, case-insensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Yakut_100_CS_AI", "Yakut-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Yakut_100_CS_AI_WS", "Yakut-100, case-sensitive, accent-insensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Yakut_100_CS_AI_KS", "Yakut-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Yakut_100_CS_AI_KS_WS", "Yakut-100, case-sensitive, accent-insensitive, kanatype-sensitive, width-sensitive"));
                _collationList.Add(new Collation("Yakut_100_CS_AS", "Yakut-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-insensitive"));
                _collationList.Add(new Collation("Yakut_100_CS_AS_WS", "Yakut-100, case-sensitive, accent-sensitive, kanatype-insensitive, width-sensitive"));
                _collationList.Add(new Collation("Yakut_100_CS_AS_KS", "Yakut-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-insensitive"));
                _collationList.Add(new Collation("Yakut_100_CS_AS_KS_WS", "Yakut-100, case-sensitive, accent-sensitive, kanatype-sensitive, width-sensitive"));
            }
            return _collationList;
        }
    }
}
